﻿/*
  E097Bの端子機能割り当て

  piocfg.hの機能を使用して端子機能の割り当てを行う。
  LPC8xxの端子機能の初期化は少々面倒なため、この初期化機能を使って簡素化。

  ・ボード上で固定されている機能のLED(PIO)・PB(PIO)・TTL/RS-485(USART)・
    IMU(I2C)・CN1(PIO/ADC)のマクロを定義
  ・LED(PIO)・TTL/RS-485(USART)・IMU(I2C)のみ端子機能の初期化用配列を
    E097B_pins に定義
  ・本ヘッダファイルを自身のソースにイクルードしコンパイルすると最低限の
    初期化処理を記述した ResetExceptionDispatch_Tail が実行される
  ・C++時はmainの前後に __libc_init_array と __libc_fini_array が呼ばれる
*/

#ifndef INC_E097B_DEF_H
#define INC_E097B_DEF_H

#ifdef __cplusplus
extern "C" {
#endif

#include  <chip.h>
#include  <piocfg.h>

// マクロ・変数など
//-----------------------------
#define _LED2   (24)  // P0_24 (ボード内)

#define _TXD0   (27)  // P0_27 USART0 TXD (ボード内)
#define _RXD0   (25)  // P0_25 USART0 RXD (ボード内)
#define _RTS0   (26)  // P0_26 USART0 RTS (ボード内)
#define _TERM   (16)  // P0_16 (ボード内)

#define _SCL0   (10)  // P0_10 I2C0 SCL (ボード内)
#define _SDA0   (11)  // P0_11 I2C0 SDA (ボード内)

#define _PB1    (5)   // P0_5 (ボード内)

#define _GPIO0  (7)   // P0_7  (CN1-7)
#define _GPIO1  (6)   // P0_6  (CN1-6)
#define _GPIO2  (14)  // P0_14 (CN1-5)
#define _GPIO3  (23)  // P0_23 (CN1-4)
#define _GPIO4  (22)  // P0_22 (CN1-3)
#define _GPIO5  (21)  // P0_21 (CN1-14)
#define _GPIO6  (20)  // P0_20 (CN1-13)
#define _GPIO7  (19)  // P0_19 (CN1-12)
#define _GPIO8  (18)  // P0_18 (CN1-11)
#define _GPIO9  (17)  // P0_17 (CN1-10)
#define _GPIO10 (13)  // P0_13 (CN1-9)
#define _GPIO11 (4)   // P0_4  (CN1-8)

#define _GPIO12 (2)   // P0_2  (CN2-3)
#define _GPIO13 (3)   // P0_3  (CN2-4)

// 端子機能設定配列
static const TPin E097B_pins[] = {
  { _LED2,   PIO_TYPE_OUTPUT_1, 0,                  PIO_MODE_DEFAULT }, // P0_24      O

  { _TXD0,   PIO_TYPE_MOVABLE,  SWM_U0_TXD_O,       PIO_MODE_DEFAULT }, // USART0 TXD O
  { _RXD0,   PIO_TYPE_MOVABLE,  SWM_U0_RXD_I,       PIO_MODE_PULLUP  }, // USART0 RXD I
  { _RTS0,   PIO_TYPE_MOVABLE,  SWM_U0_RTS_O,       PIO_MODE_DEFAULT }, // USART0 RTS O
  { _TERM,   PIO_TYPE_OUTPUT_1, 0,                  PIO_MODE_DEFAULT }, // P0_16      O

  { _SCL0,   PIO_TYPE_FIXED,    SWM_FIXED_I2C0_SCL, PIO_MODE_DEFAULT }, // I2C0 SCL
  { _SDA0,   PIO_TYPE_FIXED,    SWM_FIXED_I2C0_SDA, PIO_MODE_DEFAULT }, // I2C0 SDA
};

// カレントにある指定ファイルの埋め込みマクロ
// シンボル名でデータ本体、シンボル名に「_size_」のプレフィクスが付与されたデータのバイトサイズの各変数を合わせて生成
#define INCBIN(file, sym) asm (\
   ".section .rodata\n"                  \
   ".balign 4\n"                         \
   ".global " #sym "\n"                  \
   #sym ":\n"                            \
   ".incbin \"" file "\"\n"              \
   ".global _size_" #sym "\n"            \
   ".set _size_" #sym ", . - " #sym "\n" \
   ".balign 4\n"                         \
   ".section \".text\"\n");              \
   extern const void *sym;\
   extern const size_t *_size_##sym;

// E097Bの固定機能のみの初期化
//-----------------------------
// スタートアップルーチンから呼ばれる
void ResetExceptionDispatch_Tail (void) {
  // 電圧低下検出割り込みとリセットレベルを設定
  Chip_SYSCON_SetBODLevels (SYSCON_BODRSTLVL_3, SYSCON_BODINTVAL_LVL3);
  // 電圧低下検出リセットを有効
  Chip_SYSCON_EnableBODReset();
  // LED,USART,ターミネータ,I2Cの端子機能を初期化
  PIO_Configure (E097B_pins, PIO_LISTSIZE (E097B_pins));

  // frg0clk = frg0_src_clk / (1 + MULT / DIV)
  //         = 60,000,000 / (1 + FRG0MUL / (FRG0DIV + 1))
  //         = 60,000,000 / (1 + 35 / (39 + 1))
  //         = 60,000,000 / 1.875 = 32MHz
  // FRG0のソースとしてsys_pll0_clk(60MHz)を選択
  LPC_SYSCON->FRG0MULT = 35;
  LPC_SYSCON->FRG0DIV = 39;
  LPC_SYSCON->FRG0CLKSEL = SYSCON_FRGCLKSRC_PLL;
  // USART0のクロックソースとしてFRG0を選択
  LPC_SYSCON->UART0CLKSEL = SYSCON_FLEXCOMMCLKSELSRC_FRG0;
  // USARTライブラリにて1ビット幅を8分割で判断する設定を行っており、
  // その結果として32MHz/8=4MHzが最大ボーレートになる
  // また32MHzを超えるとCPU側のバスとのタイミングに不整合が生じる
}

// ターミネーター (GPIO)
//-----------------------------
// ターミネータのON/OFF
STATIC INLINE void SetTerminator (bool On) {
  Chip_GPIO_SetPinState (LPC_GPIO_PORT, 0, _TERM, !On);
}

// LED (GPIO)
//-----------------------------
// LEDの点灯状態反転
STATIC INLINE void ToggleLED (void) {
  Chip_GPIO_SetPinToggle (LPC_GPIO_PORT, 0, _LED2);
}

// LEDのON/OFF
STATIC INLINE void SetLED (bool On) {
  Chip_GPIO_SetPinState (LPC_GPIO_PORT, 0, _LED2, !On);
}

#ifdef __cplusplus
// グローバルクラスのコンストラクタ/デストラクタ処置
//-----------------------------
extern "C" void (*__preinit_array_start)();
extern "C" void (*__preinit_array_end)();
extern "C" void (*__init_array_start)();
extern "C" void (*__init_array_end)();
extern "C" void (*__fini_array_end)();
extern "C" void (*__fini_array_start)();

// Iterate over all the init routines.
extern "C" void __libc_init_array (void) {
  for (void (**pf)() = &__preinit_array_start; pf < &__preinit_array_end; ++pf)  (*pf)();
  for (void (**pf)() = &__init_array_start; pf < &__init_array_end; ++pf)  (*pf)();
}

// Run all the cleanup routines.
extern "C" void __libc_fini_array (void) {
  for (void (**pf)() = &__fini_array_start; pf < &__fini_array_end; ++pf)  (*pf)();
}

//extern "C" { void* __dso_handle __attribute__ ((__weak__)); }
//-fno-use-cxa-atexit

#endif

#ifdef __cplusplus
}
#endif

#endif
