﻿/*
  SPI (TAD2141)

  磁気式角度センサの角度情報をSPIを使って取得。
  https://product.tdk.com/ja/search/sensor/angle/tmr-angle/info?part_no=TAD2141-BAAA

  ・GPIO8  MIOS0
    GPIO9  MOSI0
    GPIO10 SCLK0
    GPIO11 SSEL0
  ・常々送受共にDMAを介して行ってみようと思っていたのでお試し
  ・LPC8xxのSPIはSSELを自動生成させるにはTXDATとTXDATCTLを使い分ける
    仕様のようで、GDLのSPIライブラリもそれに従っており、DMAでもそれを
    踏襲すると送信に2つのデスクリプタが必要
  ・送信用の2つのDMAデスクリプタはChip_DMA_Tableを併用するとうまくい
    かなかったので(深く追求せず)、別途追加で宣言
  ・SDKやレジスタへのアクセスが多いので、初期化時と送信時に分けて一つ
    ずつ必要の可否を試し必要最低限の記述にとどめた
  ・気のせいでなければ本コードを前提にSPIをDMAで使用すると数Mbps以上を
    指定した場合にSCKの出力が不完全になり、それに伴い読み出したデータ
    が壊れる
 */

#include  <chip.h>
// ボードの端子割り当てと最小限の初期化
#include  "e097b_def.h"

// USART_TYPEマクロで使用するUSARTと機能を選択
// CN3/CN4     1:ポーリング, 2:割り込み, 3:DMA
// CN1-6,CN1-7 4:ポーリング, 5:割り込み, 6:DMA
#ifndef USART_TYPE
  #define USART_TYPE 3
#endif
#include  "e097b_usart.h"

// SPI用関数
#include  <wl_spi.h>
#include  <wl_delay.h>

#include  <string.h>
#include  <stdlib.h>

// マクロ・グローバル変数など
//-----------------------------
// SPIの送信用DMA descriptorを2つ定義
static DMA_CHDESC_T dmaTXDescriptor[2] __attribute__ ((aligned (16)));

// SPIの初期条件(初期化の一部のみGDLのライブラリを使用)
const TSPIConf spiconf = { LPC_SPI0, SPIMode0, SPIBitMSBF, 1000000, 8, SPICSLow };

// DMA用送受信バッファ
uint8_t SPITxBuf[100], SPIRxBuf[100];
uint32_t SPITxBufLast;

// DMAによるSPI送受信
// spiconf.datawにかかわらず8ビット幅
//-----------------------------
bool SPI_WriteRead (const TSPIConf *SPIConf, const uint8_t *txd, uint8_t *rxd, uint16_t datalen) {
  if (datalen > sizeof (SPITxBuf)) return false;

  // 送信データを送信バッファへコピー
  memcpy (SPITxBuf, txd, datalen);

  // SPIの初期条件設定
  Chip_SPI_ClearStatus (SPIConf->SPIx, SPI_STAT_CLR_RXOV | SPI_STAT_CLR_TXUR | SPI_STAT_CLR_SSA | SPI_STAT_CLR_SSD);
  SPIConf->SPIx->TXCTRL =
    SPI_TXCTL_ASSERT_SSEL |   // SSELをアサート
    SPI_TXCTL_FLEN (8 - 1);   // 送信データのビット幅-1

  // 受信設定
  Chip_DMA_Table[DMAREQ_SPI0_RX].xfercfg =
    DMA_XFERCFG_CFGVALID |
    DMA_XFERCFG_WIDTH_8 |
    DMA_XFERCFG_SRCINC_0 |
    DMA_XFERCFG_DSTINC_1 |
    DMA_XFERCFG_RELOAD |
    DMA_XFERCFG_XFERCOUNT (datalen);
  Chip_DMA_Table[DMAREQ_SPI0_RX].source = DMA_ADDR (&SPIConf->SPIx->RXDAT);
  Chip_DMA_Table[DMAREQ_SPI0_RX].dest = DMA_ADDR (SPIRxBuf) + datalen - 1;

  Chip_DMA_SetupTranChannel (LPC_DMA, DMAREQ_SPI0_RX, (DMA_CHDESC_T *)&Chip_DMA_Table[DMAREQ_SPI0_RX]);
  Chip_DMA_SetupChannelTransfer (LPC_DMA, DMAREQ_SPI0_RX, Chip_DMA_Table[DMAREQ_SPI0_RX].xfercfg);
  Chip_DMA_SetValidChannel (LPC_DMA, DMAREQ_SPI0_RX);
  Chip_DMA_SWTriggerChannel (LPC_DMA, DMAREQ_SPI0_RX);

  // 最終バイトを除く送信設定
  dmaTXDescriptor[0].xfercfg =
    DMA_XFERCFG_CFGVALID |
    DMA_XFERCFG_SWTRIG |
    DMA_XFERCFG_WIDTH_8 |
    DMA_XFERCFG_SRCINC_1 |
    DMA_XFERCFG_DSTINC_0 |
    DMA_XFERCFG_RELOAD |
    DMA_XFERCFG_XFERCOUNT (datalen - 1);
  dmaTXDescriptor[0].source = DMA_ADDR (SPITxBuf) + datalen - 1 - 1;

  // 最終バイトのみの送信設定
  SPITxBufLast =
    SPI_TXDATCTL_EOF |                          // フレームの終了
    SPI_TXDATCTL_EOT |                          // 転送の終了 SSELは転送終了時にディアサート
    SPI_TXDATCTL_LEN (8 - 1) |                  // 送信データのビット幅-1
    SPI_TXDATCTL_DATA (SPITxBuf[datalen - 1]);  // 送信データ
  dmaTXDescriptor[1].xfercfg =
    DMA_XFERCFG_CFGVALID |
    DMA_XFERCFG_SWTRIG |
    DMA_XFERCFG_WIDTH_32 |
    DMA_XFERCFG_SRCINC_0 |
    DMA_XFERCFG_DSTINC_0 |
    DMA_XFERCFG_XFERCOUNT (1);

  Chip_DMA_SetupTranChannel (LPC_DMA, DMAREQ_SPI0_TX, (DMA_CHDESC_T *)&dmaTXDescriptor[0]);
  Chip_DMA_SetValidChannel (LPC_DMA, DMAREQ_SPI0_TX);

  // 送信
  Chip_DMA_SetupChannelTransfer (LPC_DMA, DMAREQ_SPI0_TX, dmaTXDescriptor[0].xfercfg);

  // 送信完了待ち
  while ((Chip_DMA_GetActiveChannels (LPC_DMA) & (1 << DMAREQ_SPI0_TX)) != 0) ;
  while (!(SPIConf->SPIx->STAT & SPI_STAT_LE));

  // 受信データをコピー
  memcpy (rxd, SPIRxBuf, datalen);

  return true;
}

// 初期化
void InitSPI (void) {
  const TPin pins_spi[] = {
    {_GPIO8, PIO_TYPE_MOVABLE,  SWM_SPI0_MISO_IO,  PIO_MODE_PULLUP },
    {_GPIO9, PIO_TYPE_MOVABLE,  SWM_SPI0_MOSI_IO,  PIO_MODE_PULLUP },
    {_GPIO10, PIO_TYPE_MOVABLE, SWM_SPI0_SCK_IO,   PIO_MODE_PULLUP },
    {_GPIO11, PIO_TYPE_MOVABLE, SWM_SPI0_SSEL0_IO, PIO_MODE_PULLUP },
  };
  PIO_Configure (pins_spi, PIO_LISTSIZE(pins_spi));

  Chip_Clock_EnablePeriphClock (SYSCON_CLOCK_SPI0);
  LPC_SYSCON->SPI0CLKSEL = SYSCON_FLEXCOMMCLKSELSRC_MAINCLK;
  // SPI.CFGとSPI.DIVのみ
  SPI_Init (&spiconf);

  // DMAの再初期化抑止
  if (!(LPC_DMA->CTRL & DMA_CTRL_ENABLE)) {
    Chip_DMA_Init (LPC_DMA);
    Chip_DMA_Enable (LPC_DMA);
    Chip_DMA_SetSRAMBase (LPC_DMA, DMA_ADDR (Chip_DMA_Table));
  }

  // DMA送信設定 (SSELを自動送出させるためだけに2つのdescripotrを使用)
  Chip_DMA_DisableChannel (LPC_DMA, DMAREQ_SPI0_TX);
  Chip_DMA_EnableChannel (LPC_DMA, DMAREQ_SPI0_TX);
  Chip_DMA_DisableIntChannel (LPC_DMA, DMAREQ_SPI0_TX);
  Chip_DMA_SetupChannelConfig (LPC_DMA, DMAREQ_SPI0_TX, (DMA_CFG_PERIPHREQEN | DMA_CFG_TRIGBURST_SNGL | DMA_CFG_CHPRIORITY (3)));
  // 最終バイトを除いたデータはTXDATを使用して送信
  //  dmaTXDescriptor[0].xfercfg =
  //  dmaTXDescriptor[0].source =
  dmaTXDescriptor[0].dest = DMA_ADDR (&spiconf.SPIx->TXDAT);
  dmaTXDescriptor[0].next = DMA_ADDR (&dmaTXDescriptor[1]); // 次のdescriptorへのリンク
  // 最終バイトのみTXDATCTLを使用してフラグ付きで送信
  //  dmaTXDescriptor[1].xfercfg = ??
  dmaTXDescriptor[1].source = DMA_ADDR (&SPITxBufLast);
  dmaTXDescriptor[1].dest = DMA_ADDR (&spiconf.SPIx->TXDATCTL);
  dmaTXDescriptor[1].next = DMA_ADDR (0);

  // DMA受信設定
  Chip_DMA_DisableChannel (LPC_DMA, DMAREQ_SPI0_RX);
  Chip_DMA_EnableChannel (LPC_DMA, DMAREQ_SPI0_RX);
  Chip_DMA_DisableIntChannel (LPC_DMA, DMAREQ_SPI0_RX);
  Chip_DMA_SetupChannelConfig (LPC_DMA, DMAREQ_SPI0_RX, (DMA_CFG_PERIPHREQEN | DMA_CFG_TRIGBURST_SNGL | DMA_CFG_CHPRIORITY (3)));
  //  Chip_DMA_Table[DMAREQ_SPI0_RX].xfercfg =
  //  Chip_DMA_Table[DMAREQ_SPI0_RX].source =
  //  Chip_DMA_Table[DMAREQ_SPI0_RX].dest =
  Chip_DMA_Table[DMAREQ_SPI0_RX].next = DMA_ADDR (0);
}

// TAD2141用サブルーチン
//-----------------------------
const uint8_t CRC8_SAE_J1850[256] = {
  0x00, 0x1D, 0x3A, 0x27, 0x74, 0x69, 0x4E, 0x53, 0xE8, 0xF5, 0xD2, 0xCF, 0x9C, 0x81, 0xA6, 0xBB,
  0xCD, 0xD0, 0xF7, 0xEA, 0xB9, 0xA4, 0x83, 0x9E, 0x25, 0x38, 0x1F, 0x02, 0x51, 0x4C, 0x6B, 0x76,
  0x87, 0x9A, 0xBD, 0xA0, 0xF3, 0xEE, 0xC9, 0xD4, 0x6F, 0x72, 0x55, 0x48, 0x1B, 0x06, 0x21, 0x3C,
  0x4A, 0x57, 0x70, 0x6D, 0x3E, 0x23, 0x04, 0x19, 0xA2, 0xBF, 0x98, 0x85, 0xD6, 0xCB, 0xEC, 0xF1,
  0x13, 0x0E, 0x29, 0x34, 0x67, 0x7A, 0x5D, 0x40, 0xFB, 0xE6, 0xC1, 0xDC, 0x8F, 0x92, 0xB5, 0xA8,
  0xDE, 0xC3, 0xE4, 0xF9, 0xAA, 0xB7, 0x90, 0x8D, 0x36, 0x2B, 0x0C, 0x11, 0x42, 0x5F, 0x78, 0x65,
  0x94, 0x89, 0xAE, 0xB3, 0xE0, 0xFD, 0xDA, 0xC7, 0x7C, 0x61, 0x46, 0x5B, 0x08, 0x15, 0x32, 0x2F,
  0x59, 0x44, 0x63, 0x7E, 0x2D, 0x30, 0x17, 0x0A, 0xB1, 0xAC, 0x8B, 0x96, 0xC5, 0xD8, 0xFF, 0xE2,
  0x26, 0x3B, 0x1C, 0x01, 0x52, 0x4F, 0x68, 0x75, 0xCE, 0xD3, 0xF4, 0xE9, 0xBA, 0xA7, 0x80, 0x9D,
  0xEB, 0xF6, 0xD1, 0xCC, 0x9F, 0x82, 0xA5, 0xB8, 0x03, 0x1E, 0x39, 0x24, 0x77, 0x6A, 0x4D, 0x50,
  0xA1, 0xBC, 0x9B, 0x86, 0xD5, 0xC8, 0xEF, 0xF2, 0x49, 0x54, 0x73, 0x6E, 0x3D, 0x20, 0x07, 0x1A,
  0x6C, 0x71, 0x56, 0x4B, 0x18, 0x05, 0x22, 0x3F, 0x84, 0x99, 0xBE, 0xA3, 0xF0, 0xED, 0xCA, 0xD7,
  0x35, 0x28, 0x0F, 0x12, 0x41, 0x5C, 0x7B, 0x66, 0xDD, 0xC0, 0xE7, 0xFA, 0xA9, 0xB4, 0x93, 0x8E,
  0xF8, 0xE5, 0xC2, 0xDF, 0x8C, 0x91, 0xB6, 0xAB, 0x10, 0x0D, 0x2A, 0x37, 0x64, 0x79, 0x5E, 0x43,
  0xB2, 0xAF, 0x88, 0x95, 0xC6, 0xDB, 0xFC, 0xE1, 0x5A, 0x47, 0x60, 0x7D, 0x2E, 0x33, 0x14, 0x09,
  0x7F, 0x62, 0x45, 0x58, 0x0B, 0x16, 0x31, 0x2C, 0x97, 0x8A, 0xAD, 0xB0, 0xE3, 0xFE, 0xD9, 0xC4,
};

// Plynormal 0x1d, Ini Value 0xff, Final xor Value 0xff
uint8_t GetCRC8 (const void *buf, size_t size) {
  uint8_t *data = (uint8_t *)buf;
  uint8_t crc8 = 0xFF;

  while (size-- != 0) crc8 = CRC8_SAE_J1850[crc8^ * data++];
  return crc8 ^ 0xff;
}

// Read agnle by streaming
bool GetStreamData (uint16_t *ang, uint8_t *stat) {
  uint8_t r[5];

  if (SPI_WriteRead (&spiconf, (const uint8_t[5]){0x05, 0x55, 0x55, 0x55, 0x55}, r, 5)) {
    if (GetCRC8 (&r[1], 3) == r[4]) {
      uint16_t *p = (uint16_t *)(void *)&r[1];
      *ang = __builtin_bswap16 (*p);
      *stat = r[3];
      return true;
    }
  }
  return false;
}

// Read value of a register
bool ReadData (uint8_t addr, uint16_t *val) {
  if (addr > 127) return false;
  uint8_t r[10];
  if (!SPI_WriteRead (&spiconf, (uint8_t[7]){0x02, addr, 0x55, 0x55, 0x55, 0x55, 0x55}, r, 7)) return false;
  for (int i = 0; i < 2; i++) if (r[2 + i] != 0x7e) return false;
  if (r[4] != 0x81) return false;
  for (int i = 0; i < 1; i++) {
    uint16_t *p = (uint16_t *)(void *)&r[3 + 2];
    val[i] = __builtin_bswap16 (*p);
  }
  return true;
}

// Writes value to register
bool WriteData (uint8_t addr, uint16_t val) {
  if (addr > 127) return false;
  uint8_t r[10];
  if (!SPI_WriteRead (&spiconf, (uint8_t[4]){0x03, addr, (val >> 8) & 0xff, val & 0xff}, r, 4)) return false;
  return true;
}

// main
//-----------------------------
int main (void) {
  static uint8_t txb[100], rxb[10];
  uint32_t cycle = 0;
  uint16_t v;
  uint8_t stat;
  char ch;

  // USART初期化
  us_init (Chip_Clock_GetFRGClockRate (0), 115200, USIP_8N1, txb, sizeof (txb), rxb, sizeof (rxb));
  us_puts ("START\n");

  // SPI初期化
  InitSPI ();

  while (1) {
    if (us_rx_buff ()) {
      ch = us_getc ();
      switch (ch) {
        case '!':
          NVIC_SystemReset ();
          break;

        // 角度取得
        case 's':
          while (!us_rx_buff ()) {
            if (GetStreamData (&v, &stat)) us_printf ("\r%6d $%02x\r", v, stat);
            Delay_ms (2);
          }
          break;
        case 'S':
          if (GetStreamData (&v, &stat)) us_printf ("%6d\n", v);
          break;

        // レジスタダンプ
        case 'd':
          us_printf ("\n");
          for (int y = 0; y < 8; y++) {
            us_printf ("$%02X:", y * 16);
            for (int x = 0; x < 16; x++) {
              if (ReadData (x + y * 8, &v)) us_printf (" %04X", v);
            }
            us_printf ("\n");
          }
          break;

        // スペアレジスタへの書き読みテスト
        case 't':
          if (WriteData (127, cycle))
            if (ReadData (127, &v))
              us_printf ("\n$7F W:%04X R:%04X\n", (uint16_t)cycle, v);
          break;
      }
    }
    cycle++;
  }
}
