﻿/*
  ID・ボーレート・動作モードの変更と動作確認

  プロトコルV2のDYNAMIXELの設定変更ツール

  ・Windows用DX2LIB向けのサンプルプログラムを移植
  ・DXMIOをホストとし、接続されたDYNAMIXELのID・ボーレート・動作モード
    をターミナルから変更
  ・TTL及びRS-485の通信ポートがDYNAMIXELとの通信で占有されてしまうため、
    GPIO経由のUSARTをデバッグコンソール化
  ・クロックの分周比の都合により1Mbpsを超えるボーレートは1M,2M,4Mbpsの
    みに対応
 */

#include  <chip.h>
// ボードの端子割り当てと最小限の初期化
#include  "e097b_def.h"

// USART_TYPEマクロで使用するUSARTと機能を選択
// CN3/CN4     1:ポーリング, 2:割り込み, 3:DMA
// CN1-6,CN1-7 4:ポーリング, 5:割り込み, 6:DMA
#define USART_TYPE 3
#include  "e097b_usart.h"

#include  <wl_delay.h>

#include  <stdio.h>
#include  <string.h>
#include  <ctype.h>

// マクロとグローバル変数
//-----------------------------
#define _TARGETID   (1)             // 対象ID
#define _BAUDRATE   (57600)         // ボーレート[bps]

// USARTによるデバッグ入出力
#if ((USART_TYPE == 4) || (USART_TYPE == 5) || (USART_TYPE == 6))
#define ENABLE_GPIO_USART
#endif
#ifdef ENABLE_GPIO_USART
  #define DEBUG_PRINTF(fmt,...) us_printf(fmt,##__VA_ARGS__)
  #define DEBUG_PRINTF_ADDR     us_printf
  #define DEBUG_GETC()          (us_rx_buff () ? us_getc () : 0)
  #define DEBUG_PUTC            us_putc
  #define DEBUG_GETS            us_gets
#else
  #define DEBUG_PRINTF(fmt,...) 
  #define DEBUG_PRINTF_ADDR     (0)
  #define DEBUG_GETC(c)         (0)
  #define DEBUG_PUTC(c)         
  #define DEBUG_GETS            
#endif

// DYNAMIXELプロトコル処理関数のプロトタイプ宣言とアイテムのアドレス定義
#include  <dx2lib.h>
#include  <dx2memmap.h>

#define _ADDR_ID        (7)
#define _ADDR_BAUDRATE  (8)

const uint32_t baudlist[7] = { 9600, 57600, 115200, 1000000, 2000000, 3000000, 4000000 };
const char *opmode[] = {
  "cur", "velo", "", "pos", "multiturn", "multiturn+cur",
  "","","","","","","","","","",
  "PWM"
};

int Getc (void) {
  return DEBUG_GETC ();
}

int Putc (int ch) {
  DEBUG_PUTC (ch);
  return 1;
}

bool InputNum (int *num) {
  char buf[100];
  DEBUG_GETS (buf, sizeof (buf));
  DEBUG_PRINTF("\n");
  return (strlen(buf) && sscanf(buf, "%d", num) == 1);
}

bool InputStr (char *str) {
  char buf[100];
  DEBUG_GETS (buf, sizeof (buf));
  DEBUG_PRINTF("\n");
  return (strlen(buf) && sscanf(buf, "%s", str) == 1);
}

bool YorN (void) {
  char ch;
  do {
    ch = Getc ();
    ch = toupper( ch );
  } while (ch == 0);
  return (ch == 'Y');
}

bool EditComAndBaudAndTarget (uint32_t *baud, uint8_t *id) {
  bool result = true;
  int n;

  DEBUG_PRINTF ("Input baudrate (default %dbps) = ", _BAUDRATE);
  if (!InputNum (&n)) { n = _BAUDRATE; result = false; }
  if (n >= 9600 && n <= 4000000) *baud = n; else *baud = _BAUDRATE;

  DEBUG_PRINTF ("Input target id (default %d) = ", _TARGETID);
  if (!InputNum (&n)) { n = _TARGETID; result = false; }
  if (n >= 1 && n < 0xfd) *id = n; else *id = _TARGETID;

  return result;
}

bool DeviceExists (TDeviceID dev, uint8_t id, uint32_t baud, uint32_t prevbaud) {
  bool detected = false;
  if (DX2_SetBaudrate (dev, baud)) {
    for (int i = 0; i < 5; i++) {
      if ((detected = DX2_Ping (dev, id, NULL))) break;
    }
  }
  DX2_SetBaudrate (dev, prevbaud);
  return detected;
}

void Prompt (uint8_t id, int32_t baud) {
  DEBUG_PRINTF ("\r\33[1m\33[0m\033[45mid:%d baud:%ld\033[49m ", id , baud);
  fflush(stdout);
}

uint32_t us0_init (uint32_t baud) {
  // DX2LIBシリアル通信用バッファ
  static uint8_t txb[100], rxb[100];
  return usart0_dma_init (Chip_Clock_GetFRGClockRate (0), baud, USIP_8N1, txb, sizeof (txb), rxb, sizeof (rxb));
}

uint32_t us0_setbaud (uint32_t baud) {
  return usart0_dma_setbaudrate (Chip_Clock_GetFRGClockRate (0), baud);
}

int main (void) {
  TDeviceID  dev;
  TErrorCode err;
  int result = 0;

  char comname[20];
  uint32_t baud;
  uint8_t id, /*btyeval, */prevbyteval;
  bool blval/*, prevblbal*/;
  double dblval, dblval_flg;
  int d;
//  uint8_t dum[1000];

#ifdef ENABLE_GPIO_USART
  // debug print用にusart3を初期化
  static uint8_t us3_txd[10], us3_rxd[10];
  us_init (Chip_Clock_GetFRGClockRate (0), 115200, USIP_8N1, us3_txd, sizeof (us3_txd), us3_rxd, sizeof (us3_rxd));
#endif

  EditComAndBaudAndTarget (&baud, &id);
  DEBUG_PRINTF("\nBaudrate=%ld, id=%d\n", baud, id);

  const TDX2Host_ConfParam conf = {
    true,                   // CRCをペリフェラルで算出するか否か
    us0_init,               // USARTの初期化関数
    us0_setbaud,            // USARTのボーレート設定関数
    usart0_dma_rx_buff,     // USARTの受信バッファ取得関数
    usart0_dma_rx_purge,    // USARTの受信バッファ破棄関数
    usart0_dma_putsbd,      // USARTの指定バイト数送信関数
    usart0_dma_getc         // USARTの1バイト受信関数
  };

  if ((dev = DX2_OpenPort (&conf, baud))) {
    Prompt (id, baud);
    while (1) {
      char ch;
      while (us_rx_buff ()) {
        Putc (ch = Getc ());
        switch (ch) {
          case 's': // scan
            DEBUG_PRINTF("\nStat scan\n");
            blval = true;
            for (int b = 0; b < 7 && blval; b++) {
              if (baudlist[b] == 3000000) b++;
              DEBUG_PRINTF("\rhost baudrate:%ldbps\n", baudlist[b]);
              DX2_SetBaudrate (dev, baudlist[b]);
              for (uint8_t i = 0; i <= 252 && blval; i++) {
                DEBUG_PRINTF (" ping to %d\r",i);
                fflush (stdout);
                if (us_rx_buff ()) {
                  Getc ();
                  blval = false;
                }
                if (DX2_Ping (dev, i, NULL)) {
                  DEBUG_PRINTF ("\r id:%3d, device name:%s \n", i, DX2_GetModelInfo (dev, i)->name);
                }
              }
            }
            DX2_SetBaudrate (dev, baud);
            break;

          case 'p': // ping
            DX2_GetModelInfo (dev, id);
            if (!DX2_GetOperatingMode (dev, id, &prevbyteval)) prevbyteval = 255;
            DEBUG_PRINTF("\nPing\nstat:%s, device name:%s, op mode:%s\n", DX2_Ping (dev, id, NULL) ? "ok" : "ng", DX2_GetModelInfo (dev, id)->name, (prevbyteval != 255) ? opmode[prevbyteval] : "?");
            break;

          case 'R': // factory reset device
            if (DX2_Ping (dev, id, NULL)) {
              DEBUG_PRINTF("\nFactory reset device\nAre you sure ?(Y/N)");
              if (YorN ()) {
                DEBUG_PRINTF("y\n");
                if (!DeviceExists (dev, 1, 57600, baud) || (id == 1 && baud ==57600)) {
                  if (DX2_Reset (dev, id, &err)) {
                    DEBUG_PRINTF("OK\n");
                    id = 1;
                    if (DX2_SetBaudrate (dev, 57600)) baud = 57600;
                  } else
                    DEBUG_PRINTF("NG: $%04x\n", err);
                } else
                  DEBUG_PRINTF("NG: That setting conflicts\n");
              } else
                DEBUG_PRINTF("n\n");
            } else
              DEBUG_PRINTF("\nCannot find device\n");
            break;

          case 'b': // change host baudrate
            DEBUG_PRINTF("\nChange host baud\nInput baudrate index\n (0:9600, 1:57600, 2:115200, 3:1M, 4:2M, 5:3M, 6:4M[bps]) = ");
            if (InputNum (&d)) {
              d = MAX (MIN (d, 6), 0);
              if (baudlist[d] == 3000000) {
                DEBUG_PRINTF("Not supported baud\n");
              } else if (DX2_SetBaudrate (dev, baudlist[d])) baud = baudlist[d];
            }
            break;
          case 'B': // change device baudrate
            if (DX2_Ping (dev, id, NULL)) {
              DEBUG_PRINTF("\nChange device baud\nInput baudrate index\n (0:9600, 1:57600, 2:115200, 3:1M, 4:2M, 5:3M, 6:4M[bps]) = ");
              if (InputNum (&d)) {
                d = MAX (MIN (d, 6), 0);
                if (baudlist[d] == 3000000) {
                  DEBUG_PRINTF("Not supported baud\n");
                } else {
                  if (!DeviceExists (dev, id, baudlist[d], baud)) {
                    if (DX2_WriteByteData (dev, id, _ADDR_BAUDRATE , d, &err)) {
                      DEBUG_PRINTF("OK\n");
                      if (DX2_SetBaudrate (dev, baudlist[d])) baud = baudlist[d];
                    } else
                      DEBUG_PRINTF("NG: $%04x\n",err);
                  } else
                    DEBUG_PRINTF("NG: That setting conflicts\n");
                }
              }
            } else
              DEBUG_PRINTF("\nCannot find devicet\n");
            break;

          case 'i': // change host id
            DEBUG_PRINTF("\nChange host id\nInput id (0..252) = ");
            if (InputNum (&d)) {
              id =  MAX (MIN (d, 252), 0);
            }
            break;
          case 'I': // change device id
            if (DX2_Ping (dev, id, NULL)) {
              DEBUG_PRINTF("\nChange device id\nInput id (0..252) = ");
              if (InputNum (&d)) {
                d = MAX (MIN (d, 252), 0);
                if (!DeviceExists (dev, d, baud, baud)) {
                  if (DX2_WriteByteData (dev, id, _ADDR_ID , d, &err)) {
                    DEBUG_PRINTF("OK\n");
                    id = d;
                  } else
                    DEBUG_PRINTF("NG: $%04x\n", err);
                } else
                  DEBUG_PRINTF("NG: That setting conflicts\n");
              }
            } else
              DEBUG_PRINTF("\nCannot find device\n");
            break;

          case 'O': // change device operating mode
            if (DX2_Ping (dev, id, NULL)) {
              DX2_InitDevicesList ();
              DX2_GetModelInfo (dev, id);
              if (DX2_GetOperatingMode (dev, id, &prevbyteval)) {
                DEBUG_PRINTF("\nChange device op mode (current op mode = %d:%s)\nInput id (0:cur 1:velo 3:pos 4:multiturn 5:multi+cur 16:PWM) = ", prevbyteval, opmode[prevbyteval]);
                if (InputNum (&d)) {
                  d = MAX (MIN (d, 16), 0);
                  if (DX2_SetOperatingMode (dev, id, d)) {
                    DX2_Reboot (dev, id, NULL); // 各設定値が残るのを嫌ってリブート
                    DEBUG_PRINTF("OK\n");
                  } else
                    DEBUG_PRINTF("NG: $%04x\n", err);
                }
              }
            } else
              DEBUG_PRINTF("\nCannot find device\n");
            break;

          case 'E': // change device torque enable
            DEBUG_PRINTF ("\n");
            if (DX2_Ping (dev, id, NULL)) {
              DX2_InitDevicesList ();
              DX2_GetModelInfo (dev, id);
              DX2_GetTorqueEnable (dev, id, &blval);
              blval = !blval;
              DEBUG_PRINTF("Change device torqule %s:%s\n", blval ? "enable" : "disable", DX2_SetTorqueEnable (dev, id, blval) ? "OK" : "NG");
            } else
              DEBUG_PRINTF("Cannot find device\n");
            break;

          case 'A':
            DEBUG_PRINTF ("\n");
            if (DX2_Ping (dev, id, NULL)) {
              DX2_InitDevicesList ();
              DX2_GetModelInfo (dev, id);
              if (DX2_GetOperatingMode (dev, id, &prevbyteval)) {
                if ((prevbyteval == 3) || (prevbyteval == 4) || (prevbyteval == 5)) {
                  DX2_GetPresentAngle (dev, id, &dblval);
                  dblval_flg = 1;
                  while (!us_rx_buff ()) {
                    dblval += (dblval_flg * 0.1);
                    if (dblval > 720.0) { dblval_flg = -1; dblval = 720.0; }
                    else if (dblval < -720.0) { dblval_flg = 1; dblval = -720.0; }
                    DEBUG_PRINTF ("goal angle:%1f[deg]      \r", dblval);
                    DX2_SetGoalAngle (dev, id, dblval);
                    Delay_ms (1);
                  }
                  Getc ();
                  DEBUG_PRINTF ("\n");
                } else
                  DEBUG_PRINTF("NG: op mode is different\n");
              }
            } else
              DEBUG_PRINTF("Cannot find device\n");
            break;

          case 'V':
            DEBUG_PRINTF ("\n");
            if (DX2_Ping (dev, id, NULL)) {
              DX2_InitDevicesList ();
              DX2_GetModelInfo (dev, id);
              if (DX2_GetOperatingMode (dev, id, &prevbyteval)) {
                if (prevbyteval == 1) {
                  DX2_GetPresentVelocity (dev, id, &dblval);
                  dblval_flg = 1;
                  while (!us_rx_buff ()) {
                    dblval += (dblval_flg * 0.1);
                    if (dblval > 720.0) { dblval_flg = -1; dblval = 720.0; }
                    else if (dblval < -720.0) { dblval_flg = 1; dblval = -720.0; }
                    DEBUG_PRINTF ("goal velocity:%2f[deg/sec]      \r",dblval);
                    DX2_SetGoalVelocity (dev, id, dblval);
                    Delay_ms (1);
                  }
                  Getc ();
                  DEBUG_PRINTF ("\n");
                } else
                  DEBUG_PRINTF("NG: op mode is different\n");
              }
            } else
              DEBUG_PRINTF("Cannot find device\n");
            break;

          case 'C':
            DEBUG_PRINTF ("\n");
            if (DX2_Ping (dev, id, NULL)) {
              DX2_InitDevicesList ();
              DX2_GetModelInfo (dev, id);
              if (DX2_GetOperatingMode (dev, id, &prevbyteval)) {
                if (prevbyteval == 0) {
                  DX2_GetPresentCurrent (dev, id, &dblval);
                  dblval_flg = 1;
                  while (!us_rx_buff ()) {
                    dblval += (dblval_flg);
                    if (dblval > 2000.0) { dblval_flg = -1; dblval = 2000.0; }
                    else if (dblval < -2000.0) { dblval_flg = 1; dblval = -2000.0; }
                    DEBUG_PRINTF (" goal current=%1f[mA]      \r",dblval);
                    DX2_SetGoalCurrent (dev, id, dblval);
                    Delay_ms (1);
                  }
                  Getc ();
                  DEBUG_PRINTF ("\n");
                } else
                  DEBUG_PRINTF("NG: op mode is different\n");
              }
            } else
              DEBUG_PRINTF("\nCannot find device\n");
            break;

          default:
            DEBUG_PRINTF("\n[s]scan [p]ping [R]factory reset device\n"
              "[b]change host baudrate [B]change device baudrate\n"
              "[i]change host id [I]change device id\n"
              "[O]change device operating mode\n"
              "[E]change device torque enable\n"
              "[A]test angle control\n"
              "[V]test velocity control\n"
              "[C]test current control\n"
            );
            break;

          case '!':
          case 'q':
          case 'Q':
            goto fin;
            break;
        }
        Prompt (id, baud);
      }
      Delay_ms (100);
    }
fin:  ;
  } else {
    DEBUG_PRINTF ("Failed to open %s\n", comname);
    result = 1;
  }

  DEBUG_PRINTF ("\nFin\n");
  Delay_ms (50);
  return result;
}
