﻿/*
  SysTick Time & Sleep

  ARMのシステムタイマを使い1kHz周期で割り込みを発生させるよう設定。
  割り込み内でカウンタをインクリメントし、1000で割り切れるタイミングで
  LEDの状態を反転。
  メインのループ内で常にSLEEPモードへの移行を励起。

  ・FROの精度が±1%なので時計としてはまず使えず、分オーダになれば積算誤
    差は歴然としてくる
  ・CMSISで提供される関数を使用してシステムタイマを初期化
  ・システムタイマはクロックと同期してカウントアップし、SysTick_Config
    の引数は-1されて再ロードレジスタに渡される
  ・再ロードレジスタは最大24ビットなので、SysTick_Configにそれを超えた
    値が指定されると失敗する
  ・SysTick_Configは成功すると0、失敗すると1を返す
  ・WFI命令で割り込み発生まで低電力モードに遷移させている
 */

#include  <chip.h>
// ボードの端子割り当てと最小限の初期化
#include  "e097b_def.h"

// USART_TYPEマクロで使用するUSARTと機能を選択
// CN3/CN4     1:ポーリング, 2:割り込み, 3:DMA
// CN1-6,CN1-7 4:ポーリング, 5:割り込み, 6:DMA
#ifndef USART_TYPE
  #define USART_TYPE 1
#endif
#include  "e097b_usart.h"

// マクロ・グローバル変数など
//-----------------------------
// カウンタ
uint32_t tickcnt;

// Interrupt handler
//-----------------------------
void SysTick_Handler (void) {
  tickcnt++;

  // 1000回に1回のタイミングでLEDの明滅を反転
  if (tickcnt % 1000 == 0) ToggleLED ();
}

// main
//-----------------------------
void main (void) {
  static uint8_t txd[20], rxd[20];

  us_init (Chip_Clock_GetFRGClockRate (0), 115200, USIP_8N1, txd, sizeof (txd), rxd, sizeof (rxd));

  // システムタイマの設定 (カウンタの初期化と再ロードレジスタの設定、割り込み許可)
  // ここでは30000000[Hz]/1000[Hz]=30000を渡す事で1000Hz周期で割り込みが発生
  if (SysTick_Config (F_CPU / 1000) == 0) {
    for (;;) {
      // 低電力モード移行
      __WFI ();
      // タイマ割り込みが発生すると復帰して以下に進む

      if (us_rx_buff ()) {
        switch (us_getc ()) {
          case '!':
            NVIC_SystemReset ();
            break;
          default:
            us_printf ("\rtickcnt=%lu        ", tickcnt);
            break;
        }
      }
    }
  }
}
