﻿/*
  SPI (WS2812B)

  少ない配線で制御できるフルカラーLED WS2812Bを光らす。
  http://www.world-semi.com/solution/list-4-1.html#108

  ・GPIO2をMOSI0に割り当て、WS2812BのINに接続
     GPIO2 --> IN
  ・WS2812Bは数百ナノ秒単位でパルス幅を判定するのと、複数接続した場合
    はある程度精度よくかつ遅滞なく生成することが求められる
  ・ネットで良く見受けられるSPIを流用して8段階という低分解能なPWMのよ
    うにON/OFF時間を生成
  ・SPIのDMAを直叩きしているのでライブラリの説明にもなってないのは悪し
    からず
  ・SPIのデータ送信中に何らかの外乱で遅滞が生じる環境では使い物になら
    なくなったため、DMAを使った一括送信に集約
  ・LEDを16個つないでテスト済み。今のところDMAの都合で42個が最大
 */

#include  <us.h>
#include  <chip.h>
// ボードの端子割り当てと最小限の初期化
#include  "e097b_def.h"
#ifndef USART_TYPE
  #define USART_TYPE 3
#endif
#include  "e097b_usart.h"

// SPI用関数
#include  <wl_spi.h>
#include  <wl_delay.h>

#include  <string.h>
#include  <stdlib.h>

// マクロ・グローバル変数など
//-----------------------------
// シリーズにつないだWS2182Bの数 (0..42)
#define _LED_NUM    (16)
// 輝度 (0..255)
#define _LUMINANCE  (32)

// 色指定
typedef struct {
  uint8_t green;  // G7..G0
  uint8_t red;    // R7..R0
  uint8_t blue;   // B7..B0
} __attribute__ ((packed)) color_t;

// マトリクスLED用 カラーテーブル
const color_t rainbow_color[9] = {
  {  0, 128,   0},  // red
  { 82, 128,   0},  // orange
  {128, 128,   0},  // yellow
  {128,   0,   0},  // green
  {128,   0, 128},  // cyan
  {  0,   0, 128},  // blue
  {  0, 128, 128},  // purple

  {128, 128, 128},  // white
  {  0,   0,   0},  // blank
};

#define _C_RED    rainbow_color[0]
#define _C_ORANGE rainbow_color[1]
#define _C_YELLOW rainbow_color[2]
#define _C_GREEN  rainbow_color[3]
#define _C_CYAN   rainbow_color[4]
#define _C_BLUE   rainbow_color[5]
#define _C_PURPLE rainbow_color[6]
#define _C_WHITE  rainbow_color[7]
#define _C_BLANK  rainbow_color[8]

// SPI送信データバッファ
// 1つのLEDが必要とするデータは8bit*3色、その1bit分のパルス幅をSPIの8bitで生成
// よって1つのLEDを光らせる時のSPIの送信データは24バイト
uint8_t SPITxBuf[_LED_NUM * 8 * 3];

// WS2812B用サブルーチン
//-----------------------------
// 輝度を色に反映
void RefLum2RGB (uint8_t lum, color_t *d, color_t s) {
  d->red = (s.red * (int)lum) >> 8;
  d->green = (s.green * (int)lum) >> 8;
  d->blue = (s.blue * (int)lum) >> 8;
}

// 1ビットを8ビットのパルス幅に変換してDMAにて送信
void SpiDmaWrite (const uint8_t *data, uint16_t datalen) {
  int len = 0;
  uint8_t mask;

  // 送信完了待ち
  while ((Chip_DMA_GetActiveChannels (LPC_DMA) & (1 << DMAREQ_SPI0_TX)) != 0) ;
  // 変換
  do {
    mask = 1 << 7;
    do {
      // 1パルス分の周期 11111111 -> 8 * 1/6MHz = 1.333 us (T0H+T0L 0.8～1.38us T1H+T1L 1.16～2us)
      // 0相当のパルス幅 11000000 -> 2 * 1/6MHz = 0.333 us (T0H 0.22～0.38us)
      // 1相当のパルス幅 11111000 -> 5 * 1/6MHz = 0.833 us (T1H 0.58～1us)
      SPITxBuf[len++] = (*data & mask) ? 0b11111000 : 0b11000000;
      mask >>= 1;
    } while (mask);
    data++;
  } while (--datalen);
  // 転送データの末尾を設定
  Chip_DMA_Table[DMAREQ_SPI0_TX].source = DMA_ADDR (SPITxBuf) + len - 1;
  // 渡されたDMA転送記述子でDMAチャネルを設定
  Chip_DMA_Table[DMAREQ_SPI0_TX].xfercfg = DMA_XFERCFG_CFGVALID | DMA_XFERCFG_SWTRIG | DMA_XFERCFG_WIDTH_8 | DMA_XFERCFG_SRCINC_1 | DMA_XFERCFG_DSTINC_0 | DMA_XFERCFG_XFERCOUNT(len);
  // DMA転送記述子でDMAチャネルを設定
  Chip_DMA_SetupTranChannel (LPC_DMA, DMAREQ_SPI0_TX, (DMA_CHDESC_T *)&Chip_DMA_Table[DMAREQ_SPI0_TX]);
  // 単一チャネルのVALIDPENDING制御ビットを設定
  Chip_DMA_SetValidChannel (LPC_DMA, DMAREQ_SPI0_TX);
  // DMAチャネル転送の設定
  Chip_DMA_SetupChannelTransfer (LPC_DMA, DMAREQ_SPI0_TX, Chip_DMA_Table[DMAREQ_SPI0_TX].xfercfg);
//    while (!(LPC_SPI0->STAT & SPI_STAT_TXRDY));
}

// LEDに反映
void UpdateLed (const color_t *data) {
  SpiDmaWrite ((uint8_t *)data, _LED_NUM * 3);
}

// 初期化
void InitSPI (void) {
  const TPin pins_spi = { _GPIO2, PIO_TYPE_MOVABLE, SWM_SPI0_MOSI_IO, PIO_MODE_DEFAULT }; // MOSIのみ使用
  PIO_Configure (&pins_spi, 1);

  // MSB First、Modeは適当、速度は6Mbps程度であれば仕様範疇
  const TSPIConf SPIConf = { LPC_SPI0, SPIMode3, SPIBitMSBF, 6000000, 8, SPICSLow };
  Chip_Clock_EnablePeriphClock (SYSCON_CLOCK_SPI0);
  LPC_SYSCON->SPI0CLKSEL = SYSCON_FLEXCOMMCLKSELSRC_MAINCLK;
  SPI_Init (&SPIConf);
  // SPI_Initが想定していないDMAを使うので追加初期化が必要
  // DMAの送受信そのものもSPIライブラリでは未対応
  LPC_SPI0->TXCTRL = SPI_TXDATCTL_RXIGNORE | SPI_TXDATCTL_LEN(8 - 1);

  // DMAの再初期化抑止
  if (!(LPC_DMA->CTRL & DMA_CTRL_ENABLE)) {
    Chip_DMA_Init (LPC_DMA);
    Chip_DMA_Enable (LPC_DMA);
    Chip_DMA_SetSRAMBase (LPC_DMA, DMA_ADDR (Chip_DMA_Table));
  }
  Chip_DMA_DisableChannel (LPC_DMA, DMAREQ_SPI0_TX);
  Chip_DMA_EnableChannel (LPC_DMA, DMAREQ_SPI0_TX);
  Chip_DMA_SetupChannelConfig (LPC_DMA, DMAREQ_SPI0_TX,(DMA_CFG_PERIPHREQEN | DMA_CFG_TRIGBURST_SNGL | DMA_CFG_CHPRIORITY(3)));
  Chip_DMA_Table[DMAREQ_SPI0_TX].dest = DMA_ADDR (&LPC_SPI0->TXDAT);
  Chip_DMA_Table[DMAREQ_SPI0_TX].next = DMA_ADDR (0);
}

// main
//-----------------------------
int main (void) {
  static uint8_t txb[100], rxb[10];
  static color_t rgb[_LED_NUM];
  char ch = '0';
  uint32_t cycle = 0;
  div_t o_t;

  // USART初期化
  us_init (Chip_Clock_GetFRGClockRate (0), 115200, USIP_8N1, txb, sizeof (txb), rxb, sizeof (rxb));
  us_puts("START\n");

  // SPI初期化
  InitSPI ();

  while (1) {
    if (us_rx_buff ()) us_putc (ch = us_getc ());

    switch (ch) {
      case '!':
        NVIC_SystemReset ();
        break;

      case '0': // 全消灯
        memset (rgb, 0, sizeof (color_t) * _LED_NUM);
        UpdateLed (rgb);
        break;
      case '1' ... '7': // 7色
        for (int i = 0; i < _LED_NUM; i++) RefLum2RGB (_LUMINANCE, &rgb[i], rainbow_color[ch - '1']);
        UpdateLed (rgb);
        break;
      case '8': // ランダム
        if (cycle % 20 == 0) {
          for (int i = 0; i < _LED_NUM; i++) RefLum2RGB (_LUMINANCE, &rgb[i], (color_t){(uint8_t)(rand() % _LUMINANCE), (uint8_t)(rand() % _LUMINANCE), (uint8_t)(rand() % _LUMINANCE)});
          UpdateLed (rgb);
        }
        break;
      case '9': // 点灯位置シフト
        o_t = div (cycle, 20);
        if (o_t.rem == 0) {
          RefLum2RGB (_LUMINANCE, &rgb[o_t.quot % _LED_NUM], _C_WHITE);
          UpdateLed (rgb);
        }
        if (cycle % 2 == 0) {
          for (int i = 0; i < _LED_NUM; i++) {
            if (rgb[i].red > 0) rgb[i].red--;
            if (rgb[i].green > 0) rgb[i].green--;
            if (rgb[i].blue > 0) rgb[i].blue--;
          }
          UpdateLed (rgb);
        }
        break;
    }
    cycle++;
    Delay_ms (5);
  }
}
