/*----------------------------------------------------------*
    b3mlib.h
    B3Mプロトコル用API V2.0
                                       Last Edit '19 12/08
   Copyright (c) 2013, 2019 BestTechnology CO.,LTD.
 *----------------------------------------------------------*/

#ifndef _B3MLIB2_H_INCLUDE
#define _B3MLIB2_H_INCLUDE

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WIN32
  #include <windows.h>
#else
  #include <pthread.h>
#endif

#ifndef _MSC_VER
  #include <stdbool.h>
#endif

#include <stdint.h>

#define BROADCASTING_ID     (0xFF)

#ifndef _WIN32
  #ifdef __MAKE_LIB__
    typedef int                             HANDLE;
    typedef pthread_mutex_t                 CRITICAL_SECTION;

    #define EnterCriticalSection(mtx)       pthread_mutex_lock(mtx)
    #define LeaveCriticalSection(mtx)       pthread_mutex_unlock(mtx)
    #define DeleteCriticalSection(mtx)      pthread_mutex_destroy(mtx)
    #define InitializeCriticalSection(mtx)  pthread_mutex_init(mtx,NULL)
    #define SetEvent(evt)                   pthread_cond_signal(evt)
    #define INVALID_int_VALUE               -1
    #define INVALID_HANDLE_VALUE            -1
  #endif
#endif

// Device ID
#ifdef _WIN32
  #ifdef _WIN64
    typedef uint64_t TDeviceID;
  #else
    typedef uint32_t TDeviceID;
  #endif
  #define _PACKED_  __attribute__((gcc_struct,__packed__))
#else
  #if defined(__x86_64__) | defined(__aarch64__ )
    typedef uint64_t TDeviceID;
    #if defined(__x86_64__)
      #define _PACKED_  __attribute__((gcc_struct,__packed__))
    #else
      #define _PACKED_  __attribute__((__packed__))
    #endif
  #else
    typedef uint32_t TDeviceID;
    #if defined(__x86__)
      #define _PACKED_  __attribute__((gcc_struct,__packed__))
    #else
      #define _PACKED_  __attribute__((__packed__))
    #endif
  #endif
#endif

// B3M option
typedef enum {OPT_LOAD = 1, OPT_SAVE = 2, OPT_READ = 3, OPT_WRITE = 4, OPT_RESET = 5, OPT_SET_POS = 6} TOption;

// Error flags
typedef uint16_t TErrorCode;
#define ERR_INVALID_DEVID   (1 << 15)
#define ERR_INVALID_ID      (1 << 14)
#define ERR_DIFF_ID         (1 << 13)
#define ERR_ILLEGAL_SIZE    (1 << 12)
#define ERR_INVALID_PARAM   (1 << 11)
#define ERR_COMM            (1 << 10)
#define ERR_CHECKSUM        (1 << 9)
#define ERR_TIMEOUT         (1 << 8)
#define ERR_NON             (0)

// デバイスの種別
typedef enum {
  devtNONE, devtB3M
} TB3ML_DevType;

// 全シリーズモデル情報
#ifdef _MSC_VER
  __pragma (pack (push, 1))
  typedef struct {
    uint32_t  modelno;          // デバイス固有のモデル番号
    char      name[16];
    TB3ML_DevType devtype;      // デバイスのグループ
    struct {                    // 位置の範囲
      int32_t max;
      int32_t min;
    }         positionlimit;
    struct {                    // 角度の範囲
      double  max;
      double  min;
    }         anglelimit;
    struct {                    // 速度の範囲
      int32_t max;
      int32_t min;
    }         velocitylimit;
    struct {                    // PWMの範囲
      int32_t max;
      int32_t min;
    }         pwmlimit;
    double    velocityratio;    // 角速度変換係数 [deg/sec]
    double    currentratio;     // 電流変換係数 [mNm]
    double    pwmratio;         // PWM変換係数 [%]
  } TB3ML_ModelInfo, *PB3ML_ModelInfo;
  __pragma (pack (pop))
#elif defined(__GNUC__)
  typedef struct {
    uint32_t  modelno;          // デバイス固有のモデル番号
    char      name[16];
    TB3ML_DevType devtype;      // デバイスのグループ
    struct {                    // 位置の範囲
      int32_t max;
      int32_t min;
    } _PACKED_ positionlimit;
    struct {                    // 角度の範囲
      double  max;
      double  min;
    } _PACKED_ anglelimit;
    struct {                    // 速度の範囲
      int32_t max;
      int32_t min;
    } _PACKED_ velocitylimit;
    struct {                    // PWMの範囲
      int32_t max;
      int32_t min;
    } _PACKED_ pwmlimit;
    double    velocityratio;    // 角速度変換係数 [deg/sec]
    double    currentratio;     // 電流変換係数 [mNm]
    double    pwmratio;         // PWM変換係数 [%]
  } _PACKED_ TB3ML_ModelInfo, *PB3ML_ModelInfo;
#endif

//
typedef struct {
  double  angle;
  double  velocity;
} TAngleVelocity, *PAngleVelocity;

typedef struct {
  double  angle;
  double  sec;
} TAngleTime, *PAngleTime;

#ifdef _MSC_VER
  __pragma (pack (push, 1))
  typedef struct {
    uint8_t id;
    int16_t pos;
  } TMultiIDPos, *PMultiIDPos;
  __pragma (pack (pop))
#elif defined(__GNUC__)
  typedef struct {
    uint8_t id;
    int16_t pos;
  } _PACKED_ TIDPos, *PIDPos;
#endif

#ifdef _WIN32
  #ifdef __cplusplus
    #ifdef __MAKE_LIB__
      #define B3MAPIDLL extern "C" __declspec(dllexport)
    #else
      #define B3MAPIDLL extern "C" __declspec(dllimport)
    #endif
  #else
    #ifdef __MAKE_LIB__
      #define B3MAPIDLL __declspec(dllexport)
    #else
      #define B3MAPIDLL __declspec(dllimport)
    #endif
  #endif
#else
  #define B3MAPIDLL
  #define WINAPI
#endif

#ifndef _DYNAMICLOAD
// When static DLL calls is used the following prototype declaration
B3MAPIDLL TDeviceID WINAPI B3M_OpenPort (char *name, uint32_t baud);
B3MAPIDLL bool WINAPI B3M_ClosePort (TDeviceID dvid);
B3MAPIDLL bool WINAPI B3M_SetBaudrate (TDeviceID dvid, uint32_t baud);
B3MAPIDLL bool WINAPI B3M_Active (TDeviceID dvid);
B3MAPIDLL void WINAPI B3M_SetTimeOutOffset (TDeviceID dvid, uint32_t t);
B3MAPIDLL double WINAPI GetQueryPerformanceCounter (void);
B3MAPIDLL bool WINAPI B3M_TxPacket (TDeviceID dvid, uint8_t id, TOption opt, uint8_t *param, uint32_t len, TErrorCode *err);
B3MAPIDLL bool WINAPI B3M_RxPacket (TDeviceID dvid, uint8_t *rdata, uint32_t rdatasize, uint32_t *rlen, uint32_t timeout, TErrorCode *err);
B3MAPIDLL bool WINAPI B3M_ReadByteData(TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdata,TErrorCode *err);
B3MAPIDLL bool WINAPI B3M_WriteByteData(TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t dat, TErrorCode *err);
B3MAPIDLL bool WINAPI B3M_ReadWordData(TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t *rdatat, TErrorCode *err);
B3MAPIDLL bool WINAPI B3M_WriteWordData(TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t dat, TErrorCode *err);
B3MAPIDLL bool WINAPI B3M_ReadLongData(TDeviceID dvid, uint8_t id, uint16_t adr, uint32_t *rdatat, TErrorCode *err);
B3MAPIDLL bool WINAPI B3M_WriteLongData(TDeviceID dvid, uint8_t id, uint16_t adr, uint32_t dat, TErrorCode *err);
B3MAPIDLL bool WINAPI B3M_ReadBlockData (TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdatat, uint32_t len, TErrorCode *err);
B3MAPIDLL bool WINAPI B3M_WriteBlockData(TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *dat, uint32_t len, TErrorCode *err);
B3MAPIDLL bool WINAPI B3M_Ping(TDeviceID dvid, uint8_t id, TErrorCode *err);
B3MAPIDLL bool WINAPI B3M_SaveROM (TDeviceID dvid, uint8_t id, TErrorCode *err);
B3MAPIDLL bool WINAPI B3M_WriteSyncData (TDeviceID dvid, uint8_t *dat, uint32_t size, TErrorCode *err);
B3MAPIDLL bool WINAPI B3M_WritePosTime (TDeviceID dvid, PIDPos pidpos, uint16_t tim, uint32_t num, TErrorCode *err);

B3MAPIDLL bool WINAPI B3ML_SetLED (TDeviceID dvid, uint8_t id, bool en);
B3MAPIDLL bool WINAPI B3ML_SetTorqueEnable (TDeviceID dvid, uint8_t id, bool en);
B3MAPIDLL bool WINAPI B3ML_SetTorqueEnables (TDeviceID dvid, const uint8_t *ids, bool *ens, int num);
B3MAPIDLL bool WINAPI B3ML_SetTorqueEnablesEquival (TDeviceID dvid, const uint8_t *ids, int num, bool en);
B3MAPIDLL bool WINAPI B3ML_GetTorqueEnable (TDeviceID dvid, uint8_t id, bool *en);
B3MAPIDLL bool WINAPI B3ML_GetTorqueEnables (TDeviceID dvid, const uint8_t *ids, bool *en, int num);
B3MAPIDLL bool WINAPI B3ML_SetGoalAngle (TDeviceID dvid, uint8_t id, double angle);
B3MAPIDLL bool WINAPI B3ML_SetGoalAngles (TDeviceID dvid, const uint8_t *ids, double *angles, int num);
B3MAPIDLL bool WINAPI B3ML_GetPresentAngle (TDeviceID dvid, uint8_t id, double *angle);
B3MAPIDLL bool WINAPI B3ML_GetPresentAngles (TDeviceID dvid, const uint8_t *ids, double *angles, int num);
B3MAPIDLL bool WINAPI B3ML_SetGoalVelocity (TDeviceID dvid, uint8_t id, double velocity);
B3MAPIDLL bool WINAPI B3ML_SetGoalVelocities (TDeviceID dvid, const uint8_t *ids, double *velocities, int num);
B3MAPIDLL bool WINAPI B3ML_GetPresentVelocity (TDeviceID dvid, uint8_t id, double *velocity);
B3MAPIDLL bool WINAPI B3ML_GetPresentVelocities (TDeviceID dvid, const uint8_t *ids, double *velocities, int num);
B3MAPIDLL bool WINAPI B3ML_SetGoalAngleAndVelocity (TDeviceID dvid, uint8_t id, double angle, double velocity);
B3MAPIDLL bool WINAPI B3ML_SetGoalAnglesAndVelocities (TDeviceID dvid, uint8_t *ids, PAngleVelocity anglevelocity, int num);
B3MAPIDLL bool WINAPI B3ML_SetGoalAngleAndTime (TDeviceID dvid, uint8_t id, double angle, double sec);
B3MAPIDLL bool WINAPI B3ML_SetGoalAnglesAndTime (TDeviceID dvid, const uint8_t *ids, double *angles, int num, double sec);
B3MAPIDLL bool WINAPI B3ML_SetGoalCurrent (TDeviceID dvid, uint8_t id, double current);
B3MAPIDLL bool WINAPI B3ML_SetGoalCurrents (TDeviceID dvid, const uint8_t *ids, double *currents, int num);
B3MAPIDLL bool WINAPI B3ML_GetPresentCurrent (TDeviceID dvid, uint8_t id, double *current);
B3MAPIDLL bool WINAPI B3ML_GetPresentCurrents (TDeviceID dvid, const uint8_t *ids, double *currents, int num);
B3MAPIDLL bool WINAPI B3ML_SetGoalPWM (TDeviceID dvid, uint8_t id, double pwm);
B3MAPIDLL bool WINAPI B3ML_SetGoalPWMs (TDeviceID dvid, const uint8_t *ids, double *pwms, int num);
B3MAPIDLL bool WINAPI B3ML_GetPresentPWM (TDeviceID dvid, uint8_t id, double *pwm);
B3MAPIDLL bool WINAPI B3ML_GetPresentPWMs (TDeviceID dvid, const uint8_t *ids, double *pwms, int num);
B3MAPIDLL bool WINAPI B3ML_SetOperatingMode (TDeviceID dvid, uint8_t id, uint8_t mode);
B3MAPIDLL bool WINAPI B3ML_SetOperatingModesEquival (TDeviceID dvid, const uint8_t *ids, int num, uint8_t mode);
B3MAPIDLL bool WINAPI B3ML_GetOperatingMode (TDeviceID dvid, uint8_t id, uint8_t *mode);
B3MAPIDLL bool WINAPI B3ML_GetHWErrorCode (TDeviceID dvid, uint8_t id, uint8_t *hwerr);
B3MAPIDLL TErrorCode WINAPI B3ML_GetErrorCode (TDeviceID dvid, uint8_t id);
B3MAPIDLL PB3ML_ModelInfo WINAPI B3ML_GetModelInfo (TDeviceID dvid, uint8_t id);
B3MAPIDLL int WINAPI B3ML_ScanDevices (TDeviceID dvid, uint8_t *ids);
B3MAPIDLL bool WINAPI B3ML_PrintDevicesList (int (*pf) (const char *, ...));
B3MAPIDLL void WINAPI B3ML_InitDevicesList (void);

#else
// When the dynamic DLL calls is used the following prototype declaration
TDeviceID (WINAPI *B3M_OpenPort)(char *name, uint32_t baud) = NULL;
bool (WINAPI *B3M_ClosePort)(TDeviceID dvid) = NULL;
bool (WINAPI *B3M_SetBaudrate) (TDeviceID dvid, uint32_t baud) = NULL;
bool (WINAPI *B3M_Active)(TDeviceID dvid) = NULL;
bool (WINAPI *B3M_SetTimeOutOffset)(TDeviceID dvid, uint32_t t) = NULL;
double (WINAPI *GetQueryPerformanceCounter) (void) = NULL;
bool (WINAPI *B3M_TxPacket) (TDeviceID dvid, uint8_t id, TOption opt, uint8_t *param, uint32_t len, TErrorCode *err) = NULL;
bool (WINAPI *B3M_RxPacket) (TDeviceID dvid, uint8_t *rdata, uint32_t rdatasize, uint32_t *rlen, uint32_t timeout, TErrorCode *err) = NULL;
bool (WINAPI *B3M_ReadByteData)(TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdatat, TErrorCode *err) = NULL;
bool (WINAPI *B3M_WriteByteData)(TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t dat, TErrorCode *err) = NULL;
bool (WINAPI *B3M_ReadWordData)(TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t *rdatat, TErrorCode *err) = NULL;
bool (WINAPI *B3M_WriteWordData)(TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t dat, TErrorCode *err) = NULL;
bool (WINAPI *B3M_ReadLongData)(TDeviceID dvid, uint8_t id, uint16_t adr, uint32_t *rdatat, TErrorCode *err) = NULL;
bool (WINAPI *B3M_WriteLongData)(TDeviceID dvid, uint8_t id, uint16_t adr, uint32_t dat, TErrorCode *err) = NULL;
bool (WINAPI *B3M_ReadBlockData)(TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdatat, uint32_t len, TErrorCode *err) = NULL;
bool (WINAPI *B3M_WriteBlockData)(TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *dat, uint32_t len, TErrorCode *err) = NULL;
bool (WINAPI *B3M_Ping)(TDeviceID dvid, uint8_t id, TErrorCode *err) = NULL;
bool (WINAPI *B3M_SaveROM) (TDeviceID dvid, uint8_t id, TErrorCode *err) = NULL;
bool (WINAPI *B3M_WriteSyncData) (TDeviceID dvid, uint8_t *dat, uint32_t size, TErrorCode *err) = NULL;
bool (WINAPI *B3M_WritePosTime) (TDeviceID dvid, PIDPos pidpos, uint16_t tim, uint32_t num, TErrorCode *err) = NULL;

bool (WINAPI *B3ML_SetLED) (TDeviceID dvid, uint8_t id, bool en) = NULL;
bool (WINAPI *B3ML_SetTorqueEnable) (TDeviceID dvid, uint8_t id, bool en) = NULL;
bool (WINAPI *B3ML_SetTorqueEnables) (TDeviceID dvid, const uint8_t *ids, bool *ens, int num) = NULL;
bool (WINAPI *B3ML_SetTorqueEnablesEquival) (TDeviceID dvid, const uint8_t *ids, int num, bool en) = NULL;
bool (WINAPI *B3ML_GetTorqueEnable) (TDeviceID dvid, uint8_t id, bool *en) = NULL;
bool (WINAPI *B3ML_GetTorqueEnables) (TDeviceID dvid, const uint8_t *ids, bool *en, int num) = NULL;
bool (WINAPI *B3ML_SetGoalAngle) (TDeviceID dvid, uint8_t id, double angle) = NULL;
bool (WINAPI *B3ML_SetGoalAngles) (TDeviceID dvid, const uint8_t *ids, double *angles, int num) = NULL;
bool (WINAPI *B3ML_GetPresentAngle) (TDeviceID dvid, uint8_t id, double *angle) = NULL;
bool (WINAPI *B3ML_GetPresentAngles) (TDeviceID dvid, const uint8_t *ids, double *angles, int num) = NULL;
bool (WINAPI *B3ML_SetGoalVelocity) (TDeviceID dvid, uint8_t id, double velocity) = NULL;
bool (WINAPI *B3ML_SetGoalVelocities) (TDeviceID dvid, const uint8_t *ids, double *velocities, int num) = NULL;
bool (WINAPI *B3ML_GetPresentVelocity) (TDeviceID dvid, uint8_t id, double *velocity) = NULL;
bool (WINAPI *B3ML_GetPresentVelocities) (TDeviceID dvid, const uint8_t *ids, double *velocities, int num) = NULL;
bool (WINAPI *B3ML_SetGoalAngleAndVelocity) (TDeviceID dvid, uint8_t id, double angle, double velocity) = NULL;
bool (WINAPI *B3ML_SetGoalAnglesAndVelocities) (TDeviceID dvid, uint8_t *ids, PAngleVelocity anglevelocity, int num) = NULL;
bool (WINAPI *B3ML_SetGoalAngleAndTime) (TDeviceID dvid, uint8_t id, double angle, double sec) = NULL;
bool (WINAPI *B3ML_SetGoalAnglesAndTime) (TDeviceID dvid, const uint8_t *ids, double *angles, int num, double sec) = NULL;
bool (WINAPI *B3ML_SetGoalCurrent) (TDeviceID dvid, uint8_t id, double current) = NULL;
bool (WINAPI *B3ML_SetGoalCurrents) (TDeviceID dvid, const uint8_t *ids, double *currents, int num) = NULL;
bool (WINAPI *B3ML_GetPresentCurrent) (TDeviceID dvid, uint8_t id, double *current) = NULL;
bool (WINAPI *B3ML_GetPresentCurrents) (TDeviceID dvid, const uint8_t *ids, double *currents, int num) = NULL;
bool (WINAPI *B3ML_SetGoalPWM) (TDeviceID dvid, uint8_t id, double pwm) = NULL;
bool (WINAPI *B3ML_SetGoalPWMs) (TDeviceID dvid, const uint8_t *ids, double *pwms, int num) = NULL;
bool (WINAPI *B3ML_GetPresentPWM) (TDeviceID dvid, uint8_t id, double *pwm) = NULL;
bool (WINAPI *B3ML_GetPresentPWMs) (TDeviceID dvid, const uint8_t *ids, double *pwms, int num) = NULL;
bool (WINAPI *B3ML_SetOperatingMode) (TDeviceID dvid, uint8_t id, uint8_t mode) = NULL;
bool (WINAPI *B3ML_SetOperatingModesEquival) (TDeviceID dvid, const uint8_t *ids, int num, uint8_t mode) = NULL;
bool (WINAPI *B3ML_GetOperatingMode) (TDeviceID dvid, uint8_t id, uint8_t *mode) = NULL;
bool (WINAPI *B3ML_GetHWErrorCode) (TDeviceID dvid, uint8_t id, uint8_t *hwerr) = NULL;
TErrorCode (WINAPI *B3ML_GetErrorCode) (TDeviceID dvid, uint8_t id) = NULL;
PB3ML_ModelInfo (WINAPI *B3ML_GetModelInfo) (TDeviceID dvid, uint8_t id) = NULL;
int (WINAPI *B3ML_ScanDevices) (TDeviceID dvid, uint8_t *ids) = NULL;
bool (WINAPI *B3ML_PrintDevicesList) (int (*pf) (const char *, ...)) = NULL;
void (WINAPI *B3ML_InitDevicesList) (void) = NULL;

static HMODULE hModule = NULL;

#ifdef _WIN32
  #ifdef _WIN64
    #define DLLNAME   "b3mlib_x64.dll"
  #else
    #define DLLNAME   "b3mlib_x32.dll"
  #endif
#else
  #ifdef __x86_64__
    #define DLLNAME   "b3mlib_x64.dll"
  #else
    #define DLLNAME   "b3mlib_x32.dll"
  #endif
#endif

// Load the DLL
bool LoadDLL (void) {
  hModule = LoadLibrary (DLLNAME);
  if (hModule == NULL) {
    return false;
  }

  *(FARPROC*)&B3M_OpenPort               = GetProcAddress (hModule, "B3M_OpenPort");
  *(FARPROC*)&B3M_ClosePort              = GetProcAddress (hModule, "B3M_ClosePort");
  *(FARPROC*)&B3M_SetBaudrate            = GetProcAddress (hModule, "B3M_SetBaudrate");
  *(FARPROC*)&B3M_Active                 = GetProcAddress (hModule, "B3M_Active");
  *(FARPROC*)&B3M_SetTimeOutOffset       = GetProcAddress (hModule, "B3M_SetTimeOutOffset");
  *(FARPROC*)&GetQueryPerformanceCounter = GetProcAddress (hModule, "GetQueryPerformanceCounter");
  *(FARPROC*)&B3M_TxPacket               = GetProcAddress (hModule, "B3M_TxPacket");
  *(FARPROC*)&B3M_RxPacket               = GetProcAddress (hModule, "B3M_RxPacket");
  *(FARPROC*)&B3M_ReadByteData           = GetProcAddress (hModule, "B3M_ReadByteData");
  *(FARPROC*)&B3M_WriteByteData          = GetProcAddress (hModule, "B3M_WriteByteData");
  *(FARPROC*)&B3M_ReadWordData           = GetProcAddress (hModule, "B3M_ReadWordData");
  *(FARPROC*)&B3M_WriteWordData          = GetProcAddress (hModule, "B3M_WriteWordData");
  *(FARPROC*)&B3M_ReadLongData           = GetProcAddress (hModule, "B3M_ReadLongData");
  *(FARPROC*)&B3M_WriteLongData          = GetProcAddress (hModule, "B3M_WriteLongData");
  *(FARPROC*)&B3M_ReadBlockData          = GetProcAddress (hModule, "B3M_ReadBlockData");
  *(FARPROC*)&B3M_WriteBlockData         = GetProcAddress (hModule, "B3M_WriteBlockData");
  *(FARPROC*)&B3M_WriteSyncData          = GetProcAddress (hModule, "B3M_WriteSyncData");
  *(FARPROC*)&B3M_Ping                   = GetProcAddress (hModule, "B3M_Ping");
  *(FARPROC*)&B3M_SaveROM                = GetProcAddress (hModule, "B3M_SaveROM");
  *(FARPROC*)&B3M_WritePosTime           = GetProcAddress (hModule, "B3M_WritePosTime");

  *(FARPROC*)&B3ML_SetTorqueEnable           = GetProcAddress (hModule, "B3ML_SetTorqueEnable");
  *(FARPROC*)&B3ML_SetTorqueEnables          = GetProcAddress (hModule, "B3ML_SetTorqueEnables");
  *(FARPROC*)&B3ML_SetTorqueEnablesEquival   = GetProcAddress (hModule, "B3ML_SetTorqueEnablesEquival");
  *(FARPROC*)&B3ML_GetTorqueEnable           = GetProcAddress (hModule, "B3ML_GetTorqueEnable");
  *(FARPROC*)&B3ML_GetTorqueEnables          = GetProcAddress (hModule, "B3ML_GetTorqueEnables");
  *(FARPROC*)&B3ML_SetLED                    = GetProcAddress (hModule, "B3ML_SetLED");
  *(FARPROC*)&B3ML_SetGoalAngle              = GetProcAddress (hModule, "B3ML_SetGoalAngle");
  *(FARPROC*)&B3ML_SetGoalAngles             = GetProcAddress (hModule, "B3ML_SetGoalAngles");
  *(FARPROC*)&B3ML_GetPresentAngle           = GetProcAddress (hModule, "B3ML_GetPresentAngle");
  *(FARPROC*)&B3ML_GetPresentAngles          = GetProcAddress (hModule, "B3ML_GetPresentAngles");
  *(FARPROC*)&B3ML_SetGoalVelocity           = GetProcAddress (hModule, "B3ML_SetGoalVelocity");
  *(FARPROC*)&B3ML_SetGoalVelocities         = GetProcAddress (hModule, "B3ML_SetGoalVelocities");
  *(FARPROC*)&B3ML_GetPresentVelocity        = GetProcAddress (hModule, "B3ML_GetPresentVelocity");
  *(FARPROC*)&B3ML_GetPresentVelocities      = GetProcAddress (hModule, "B3ML_GetPresentVelocities");
  *(FARPROC*)&B3ML_SetGoalAngleAndVelocity   = GetProcAddress (hModule, "B3ML_SetGoalAngleAndVelocity");
  *(FARPROC*)&B3ML_SetGoalAnglesAndVelocities= GetProcAddress (hModule, "B3ML_SetGoalAnglesAndVelocities");
  *(FARPROC*)&B3ML_SetGoalAngleAndTime       = GetProcAddress (hModule, "B3ML_SetGoalAngleAndTime");
  *(FARPROC*)&B3ML_SetGoalAnglesAndTime      = GetProcAddress (hModule, "B3ML_SetGoalAnglesAndTime");
  *(FARPROC*)&B3ML_SetGoalPWM                = GetProcAddress (hModule, "B3ML_SetGoalPWM");
  *(FARPROC*)&B3ML_SetGoalPWMs               = GetProcAddress (hModule, "B3ML_SetGoalPWMs");
  *(FARPROC*)&B3ML_GetPresentPWM             = GetProcAddress (hModule, "B3ML_GetPresentPWM");
  *(FARPROC*)&B3ML_GetPresentPWMs            = GetProcAddress (hModule, "B3ML_GetPresentPWMs");
  *(FARPROC*)&B3ML_SetGoalCurrent            = GetProcAddress (hModule, "B3ML_SetGoalCurrent");
  *(FARPROC*)&B3ML_SetGoalCurrents           = GetProcAddress (hModule, "B3ML_SetGoalCurrents");
  *(FARPROC*)&B3ML_GetPresentCurrent         = GetProcAddress (hModule, "B3ML_GetPresentCurrent");
  *(FARPROC*)&B3ML_GetPresentCurrents        = GetProcAddress (hModule, "B3ML_GetPresentCurrents");
  *(FARPROC*)&B3ML_SetOperatingMode          = GetProcAddress (hModule, "B3ML_SetOperatingMode");
  *(FARPROC*)&B3ML_SetOperatingModesEquival  = GetProcAddress (hModule, "B3ML_SetOperatingModesEquival");
  *(FARPROC*)&B3ML_GetOperatingMode          = GetProcAddress (hModule, "B3ML_GetOperatingMode");
  *(FARPROC*)&B3ML_GetHWErrorCode            = GetProcAddress (hModule, "B3ML_GetHWErrorCode");
  *(FARPROC*)&B3ML_GetErrorCode              = GetProcAddress (hModule, "B3ML_GetErrorCode");
  *(FARPROC*)&B3ML_GetModelInfo              = GetProcAddress (hModule, "B3ML_GetModelInfo");
  *(FARPROC*)&B3ML_ScanDevices               = GetProcAddress (hModule, "B3ML_ScanDevices");
  *(FARPROC*)&B3ML_PrintDevicesList          = GetProcAddress (hModule, "B3ML_PrintDevicesList");
  *(FARPROC*)&B3ML_InitDevicesList           = GetProcAddress (hModule, "B3ML_InitDevicesList");

  return
    (B3M_OpenPort               != NULL) &&
    (B3M_ClosePort              != NULL) &&
    (B3M_SetBaudrate            != NULL) &&
    (B3M_Active                 != NULL) &&
    (B3M_SetTimeOutOffset       != NULL) &&
    (GetQueryPerformanceCounter != NULL) &&
    (B3M_TxPacket               != NULL) &&
    (B3M_RxPacket               != NULL) &&
    (B3M_ReadByteData           != NULL) &&
    (B3M_WriteByteData          != NULL) &&
    (B3M_ReadWordData           != NULL) &&
    (B3M_WriteWordData          != NULL) &&
    (B3M_ReadLongData           != NULL) &&
    (B3M_WriteLongData          != NULL) &&
    (B3M_ReadBlockData          != NULL) &&
    (B3M_WriteBlockData         != NULL) &&
    (B3M_WriteSyncData          != NULL) &&
    (B3M_Ping                   != NULL) &&
    (B3M_SaveROM                != NULL) &&
    (B3M_WritePosTime           != NULL) &&

    (B3ML_SetTorqueEnable           != NULL) &&
    (B3ML_SetTorqueEnables          != NULL) &&
    (B3ML_SetTorqueEnablesEquival   != NULL) &&
    (B3ML_GetTorqueEnable           != NULL) &&
    (B3ML_GetTorqueEnables          != NULL) &&
    (B3ML_SetLED                    != NULL) &&
    (B3ML_SetGoalAngle              != NULL) &&
    (B3ML_SetGoalAngles             != NULL) &&
    (B3ML_GetPresentAngle           != NULL) &&
    (B3ML_GetPresentAngles          != NULL) &&
    (B3ML_SetGoalVelocity           != NULL) &&
    (B3ML_SetGoalVelocities         != NULL) &&
    (B3ML_GetPresentVelocity        != NULL) &&
    (B3ML_GetPresentVelocities      != NULL) &&
    (B3ML_SetGoalAngleAndVelocity   != NULL) &&
    (B3ML_SetGoalAnglesAndVelocities!= NULL) &&
    (B3ML_SetGoalAngleAndTime       != NULL) &&
    (B3ML_SetGoalAnglesAndTime      != NULL) &&
    (B3ML_SetGoalCurrent            != NULL) &&
    (B3ML_SetGoalCurrents           != NULL) &&
    (B3ML_GetPresentCurrent         != NULL) &&
    (B3ML_GetPresentCurrents        != NULL) &&
    (B3ML_SetGoalPWM                != NULL) &&
    (B3ML_SetGoalPWMs               != NULL) &&
    (B3ML_GetPresentPWM             != NULL) &&
    (B3ML_GetPresentPWMs            != NULL) &&
    (B3ML_SetOperatingMode          != NULL) &&
    (B3ML_SetOperatingModesEquival  != NULL) &&
    (B3ML_GetOperatingMode          != NULL) &&
    (B3ML_GetHWErrorCode            != NULL) &&
    (B3ML_GetErrorCode              != NULL) &&
    (B3ML_GetModelInfo              != NULL) &&
    (B3ML_ScanDevices               != NULL) &&
    (B3ML_PrintDevicesList          != NULL) &&
    (B3ML_InitDevicesList           != NULL);
}

// Unload the DLL
void UnloadDLL (void) {
  if (hModule != NULL) {
    FreeLibrary (hModule);
    B3M_OpenPort               = NULL;
    B3M_ClosePort              = NULL;
    B3M_SetBaudrate            = NULL;
    B3M_Active                 = NULL;
    B3M_SetTimeOutOffset       = NULL;
    GetQueryPerformanceCounter = NULL;
    B3M_TxPacket               = NULL;
    B3M_RxPacket               = NULL;
    B3M_ReadByteData           = NULL;
    B3M_WriteByteData          = NULL;
    B3M_ReadWordData           = NULL;
    B3M_WriteWordData          = NULL;
    B3M_ReadLongData           = NULL;
    B3M_WriteLongData          = NULL;
    B3M_ReadBlockData          = NULL;
    B3M_WriteBlockData         = NULL;
    B3M_WriteSyncData          = NULL;
    B3M_Ping                   = NULL;
    B3M_SaveROM                = NULL;
    B3M_WritePosTime           = NULL;

    B3ML_SetTorqueEnable           = NULL;
    B3ML_SetTorqueEnables          = NULL;
    B3ML_SetTorqueEnablesEquival   = NULL;
    B3ML_GetTorqueEnable           = NULL;
    B3ML_GetTorqueEnables          = NULL;
    B3ML_SetLED                    = NULL;
    B3ML_SetGoalAngle              = NULL;
    B3ML_SetGoalAngles             = NULL;
    B3ML_GetPresentAngle           = NULL;
    B3ML_GetPresentAngles          = NULL;
    B3ML_SetGoalVelocity           = NULL;
    B3ML_SetGoalVelocities         = NULL;
    B3ML_GetPresentVelocity        = NULL;
    B3ML_GetPresentVelocities      = NULL;
    B3ML_SetGoalAngleAndVelocity   = NULL;
    B3ML_SetGoalAnglesAndVelocities= NULL;
    B3ML_SetGoalAngleAndTime       = NULL;
    B3ML_SetGoalAnglesAndTime      = NULL;
    B3ML_SetGoalCurrent            = NULL;
    B3ML_SetGoalCurrents           = NULL;
    B3ML_GetPresentCurrent         = NULL;
    B3ML_GetPresentCurrents        = NULL;
    B3ML_SetGoalPWM                = NULL;
    B3ML_SetGoalPWMs               = NULL;
    B3ML_GetPresentPWM             = NULL;
    B3ML_GetPresentPWMs            = NULL;
    B3ML_SetOperatingMode          = NULL;
    B3ML_SetOperatingModesEquival  = NULL;
    B3ML_GetOperatingMode          = NULL;
    B3ML_GetHWErrorCode            = NULL;
    B3ML_GetErrorCode              = NULL;
    B3ML_GetModelInfo              = NULL;
    B3ML_ScanDevices               = NULL;
    B3ML_PrintDevicesList          = NULL;
    B3ML_InitDevicesList           = NULL;
  }
}

#endif

#ifdef __cplusplus
}
#endif

#endif //_B3MLIB2_H_INCLUDE
