/*----------------------------------------------------------*
    icslib.h
    ICSプロトコル用API V2.0
                                       Last Edit '18 1/9
   Copyright (c) 2005, 2018 BestTechnology CO.,LTD.
 *----------------------------------------------------------*/

#ifndef _ICSLIB_H_INCLUDE
#define _ICSLIB_H_INCLUDE

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WIN32
  #include <windows.h>
#else
  #include <pthread.h>
#endif

#ifndef _MSC_VER
  #include <stdbool.h>
#endif

#include <stdint.h>

#define BROADCASTING_ID     (0xfe)

#ifndef _WIN32
  #ifdef __MAKE_LIB__
    typedef int                             HANDLE;
    typedef pthread_mutex_t                 CRITICAL_SECTION;

    #define EnterCriticalSection(mtx)       pthread_mutex_lock(mtx)
    #define LeaveCriticalSection(mtx)       pthread_mutex_unlock(mtx)
    #define DeleteCriticalSection(mtx)      pthread_mutex_destroy(mtx)
    #define InitializeCriticalSection(mtx)  pthread_mutex_init(mtx,NULL)
    #define SetEvent(evt)                   pthread_cond_signal(evt)
    #define INVALID_int_VALUE               -1
    #define INVALID_HANDLE_VALUE            -1
  #endif
#endif

// Device ID
#ifdef _WIN32
  #ifdef _WIN64
    typedef uint64_t TDeviceID;
  #else
    typedef uint32_t TDeviceID;
  #endif
#else
  #ifdef __x86_64__
    typedef uint64_t TDeviceID;
  #else
    typedef uint32_t TDeviceID;
  #endif
#endif

// Kind of ICS command
typedef enum {CMD_WRITE_POS = 4, CMD_READ_PARAM = 5, CMD_WRITE_PARAM = 6, CMD_ID = 7} TCommand;

// ICS sub command
#define SC_EEPROM         (0)
#define SC_STRETCH        (1)
#define SC_SPEED          (2)
#define SC_CURRENT        (3)
#define SC_TEMPTURE       (4)
#define SC_POSITION       (5)

#define SC_READ_ID        (0)
#define SC_WRITE_ID       (1)

// Error flags
typedef uint16_t TErrorCode;
#define ERR_INVALID_DEVID   (1 << 15)
#define ERR_INVALID_ID      (1 << 14)
#define ERR_DIFF_ID         (1 << 13)
#define ERR_ILLEGAL_SIZE    (1 << 12)
#define ERR_INVALID_PARAM   (1 << 11)
#define ERR_COMM            (1 << 10)
#define ERR_CHECKSUM        (1 << 9)
#define ERR_TIMEOUT         (1 << 8)
#define ERR_ICS_INST        (1 << 6)
#define ERR_ICS_OVERLOAD    (1 << 5)
#define ERR_ICS_CHECKSUM    (1 << 4)
#define ERR_ICS_RANGE       (1 << 3)
#define ERR_ICS_OVERHEAT    (1 << 2)
#define ERR_ICS_ANGLE       (1 << 1)
#define ERR_ICS_OVERVOLTAGE (1 << 0)
#define ERR_NON             (0)

// デバイスの種別
typedef enum {
  devtNONE, devtICS20, devtICS30, devtICS35, devtICS36
} TICSL_DevType;

// 全シリーズモデル情報
#ifdef _MSC_VER
  __pragma (pack (push, 1))
  typedef struct {
    uint16_t    modelno;        // デバイス固有のモデル番号
    char        name[16];
    TICSL_DevType devtype;       // デバイスのグループ
    struct {                    // 位置の範囲
      int32_t max;
      int32_t min;
    }           positionlimit;
    struct {                    // 角度の範囲
      double    max;
      double    min;
    }           anglelimit;
    struct {                    // 速度の範囲
      int32_t max;
      int32_t min;
    }           velocitylimit;
    struct {                    // PWMの範囲
      int32_t max;
      int32_t min;
    }           pwmlimit;
    double      velocityratio;  // 角速度変換係数 [deg/sec]
    double      currentratio;   // 電流変換係数 [mA]
    double      pwmratio;       // PWM変換係数 [%]
  } TICSL_ModelInfo, *PICSL_ModelInfo;
  __pragma (pack (pop))
#elif defined(__GNUC__)
  typedef struct {
    uint16_t    modelno;        // デバイス固有のモデル番号
    char        name[16];
    TICSL_DevType devtype;       // デバイスのグループ
    struct {                    // 位置の範囲
      int32_t max;
      int32_t min;
    } __attribute__ ((gcc_struct, __packed__)) positionlimit;
    struct {                    // 角度の範囲
      double    max;
      double    min;
    } __attribute__ ((gcc_struct, __packed__)) anglelimit;
    struct {                    // 速度の範囲
      int32_t max;
      int32_t min;
    } __attribute__ ((gcc_struct, __packed__)) velocitylimit;
    struct {                    // PWMの範囲
      int32_t max;
      int32_t min;
    } __attribute__ ((gcc_struct, __packed__)) pwmlimit;
    double      velocityratio;  // 角速度変換係数 [deg/sec]
    double      currentratio;   // 電流変換係数 [mA]
    double      pwmratio;       // PWM変換係数 [%]
  } __attribute__ ((gcc_struct, __packed__)) TICSL_ModelInfo, *PICSL_ModelInfo;
#endif

//
typedef struct {
  double  angle;
  double  velocity;
} TAngleVelocity, *PAngleVelocity;

#ifdef _WIN32
  #ifdef __cplusplus
    #ifdef __MAKE_LIB__
      #define ICSAPIDLL extern "C" __declspec(dllexport)
    #else
      #define ICSAPIDLL extern "C" __declspec(dllimport)
    #endif
  #else
    #ifdef __MAKE_LIB__
      #define ICSAPIDLL __declspec(dllexport)
    #else
      #define ICSAPIDLL __declspec(dllimport)
    #endif
  #endif
#else
  #define ICSAPIDLL
  #define WINAPI
#endif

#ifndef _DYNAMICLOAD
// When static DLL calls is used the following prototype declaration
ICSAPIDLL TDeviceID WINAPI ICS_OpenPort (char *name, uint32_t baud);
ICSAPIDLL bool WINAPI ICS_ClosePort (TDeviceID dvid);
ICSAPIDLL void WINAPI ICS_SetEchoCancel (bool becho);
ICSAPIDLL bool WINAPI ICS_SetBaudrate (TDeviceID dvid, uint32_t baud);
ICSAPIDLL bool WINAPI ICS_Active (TDeviceID dvid);
ICSAPIDLL void WINAPI ICS_SetTimeOutOffset (TDeviceID dvid, uint32_t t);
ICSAPIDLL double WINAPI GetQueryPerformanceCounter (void);
ICSAPIDLL bool WINAPI ICS_TxPacket (TDeviceID dvid, uint8_t id, TCommand cmd, const uint8_t *param, uint32_t len, TErrorCode *err);
ICSAPIDLL bool WINAPI ICS_RxPacket (TDeviceID dvid, uint8_t *rdata, uint32_t rdatasize, uint32_t *rlen, uint32_t timeout, TErrorCode *err);
ICSAPIDLL bool WINAPI ICS_ReadByteData(TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdata, TErrorCode *errcode);
ICSAPIDLL bool WINAPI ICS_WriteByteData(TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t dat, TErrorCode *errcode);
ICSAPIDLL bool WINAPI ICS_ReadWordData(TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t *rdata, TErrorCode *errcode);
ICSAPIDLL bool WINAPI ICS_WriteWordData(TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t dat, TErrorCode *errcode);
ICSAPIDLL bool WINAPI ICS_ReadBlockData (TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdata, uint32_t len, TErrorCode *errcode);
ICSAPIDLL bool WINAPI ICS_WriteBlockData(TDeviceID dvid, uint8_t id, uint16_t adr, const uint8_t *dat, uint32_t len, TErrorCode *errcode);
ICSAPIDLL bool WINAPI ICS_WriteSyncData (TDeviceID dvid, uint8_t *dat, uint32_t size, TErrorCode *errcode);
ICSAPIDLL bool WINAPI ICS_Ping(TDeviceID dvid, uint8_t id, TErrorCode *errcode);

ICSAPIDLL void WINAPI ics_purge (TDeviceID dvid);
ICSAPIDLL bool WINAPI ics_puts (TDeviceID dvid, uint8_t *c, int len);
ICSAPIDLL bool WINAPI ics_gets (TDeviceID d, uint8_t *buf, int bytetoread, int *bytesread, double tout);

ICSAPIDLL bool WINAPI ICSL_SetLED (TDeviceID dvid, uint8_t id, bool en);
ICSAPIDLL bool WINAPI ICSL_SetTorqueEnable (TDeviceID dvid, uint8_t id, bool en);
ICSAPIDLL bool WINAPI ICSL_SetTorqueEnables (TDeviceID dvid, const uint8_t *ids, bool *ens, int num);
ICSAPIDLL bool WINAPI ICSL_SetTorqueEnablesEquival (TDeviceID dvid, const uint8_t *ids, int num, bool en);
ICSAPIDLL bool WINAPI ICSL_SetGoalAngle (TDeviceID dvid, uint8_t id, double angle);
ICSAPIDLL bool WINAPI ICSL_SetGoalAngles (TDeviceID dvid, const uint8_t *ids, double *angles, int num);
ICSAPIDLL bool WINAPI ICSL_GetPresentAngle (TDeviceID dvid, uint8_t id, double *angle);
ICSAPIDLL bool WINAPI ICSL_GetPresentAngles (TDeviceID dvid, const uint8_t *ids, double *angles, int num);
ICSAPIDLL bool WINAPI ICSL_SetGoalVelocity (TDeviceID dvid, uint8_t id, double velocity);
ICSAPIDLL bool WINAPI ICSL_SetGoalVelocitys (TDeviceID dvid, const uint8_t *ids, double *velocitys, int num);
ICSAPIDLL bool WINAPI ICSL_GetPresentVelocity (TDeviceID dvid, uint8_t id, double *velocity);
ICSAPIDLL bool WINAPI ICSL_GetPresentVelocitys (TDeviceID dvid, const uint8_t *ids, double *velocity, int num);
ICSAPIDLL bool WINAPI ICSL_SetGoalAngleAndVelocity (TDeviceID dvid, uint8_t id, double angle, double velocity);
ICSAPIDLL bool WINAPI ICSL_SetGoalAnglesAndVelocitys (TDeviceID dvid, uint8_t *ids, PAngleVelocity anglevelocity, int num);
ICSAPIDLL bool WINAPI ICSL_SetGoalAngleAndTime (TDeviceID dvid, uint8_t id, double angle, double sec);
ICSAPIDLL bool WINAPI ICSL_SetGoalAnglesAndTime (TDeviceID dvid, const uint8_t *ids, double *angles, int num, double sec);
ICSAPIDLL bool WINAPI ICSL_SetGoalCurrent (TDeviceID dvid, uint8_t id, double current);
ICSAPIDLL bool WINAPI ICSL_SetGoalCurrents (TDeviceID dvid, const uint8_t *ids, double *currents, int num);
ICSAPIDLL bool WINAPI ICSL_GetPresentCurrent (TDeviceID dvid, uint8_t id, double *current);
ICSAPIDLL bool WINAPI ICSL_GetPresentCurrents (TDeviceID dvid, const uint8_t *ids, double *currents, int num);
ICSAPIDLL bool WINAPI ICSL_SetGoalPWM (TDeviceID dvid, uint8_t id, double pwm);
ICSAPIDLL bool WINAPI ICSL_SetGoalPWMs (TDeviceID dvid, const uint8_t *ids, double *pwms, int num);
ICSAPIDLL bool WINAPI ICSL_GetPresentPWM (TDeviceID dvid, uint8_t id, double *pwm);
ICSAPIDLL bool WINAPI ICSL_GetPresentPWMs (TDeviceID dvid, const uint8_t *ids, double *pwms, int num);
ICSAPIDLL bool WINAPI ICSL_SetOperatingMode (TDeviceID dvid, uint8_t id, uint8_t mode);
ICSAPIDLL bool WINAPI ICSL_SetOperatingModesEquival (TDeviceID dvid, const uint8_t *ids, int num, uint8_t mode);
ICSAPIDLL bool WINAPI ICSL_GetOperatingMode (TDeviceID dvid, uint8_t id, uint8_t *mode);
ICSAPIDLL bool WINAPI ICSL_GetHWErrorCode (TDeviceID dvid, uint8_t id, uint8_t *hwerr);
ICSAPIDLL TErrorCode WINAPI ICSL_GetErrorCode (TDeviceID dvid, uint8_t id);
ICSAPIDLL PICSL_ModelInfo WINAPI ICSL_GetModelInfo (TDeviceID dvid, uint8_t id);
ICSAPIDLL PICSL_ModelInfo WINAPI ICSL_ForceSetModelInfo (TDeviceID dvid, uint8_t id, TICSL_DevType devtype);
ICSAPIDLL int WINAPI ICSL_ScanDevices (TDeviceID dvid, uint8_t *ids);
ICSAPIDLL bool WINAPI ICSL_PrintDevicesList (int *pf (const char *, ...));
ICSAPIDLL void WINAPI ICSL_InitDevicesList (void);

#else
// When the dynamic DLL calls is used the following prototype declaration
TDeviceID (WINAPI *ICS_OpenPort)(char *name, long baud) = NULL;
bool (WINAPI *ICS_ClosePort)(TDeviceID dvid) = NULL;
void (WINAPI *ICS_SetEchoCancel)(bool becho) = NULL;
bool (WINAPI *ICS_SetBaudrate) (TDeviceID dvid, long baud) = NULL;
bool (WINAPI *ICS_Active)(TDeviceID dvid) = NULL;
void (WINAPI *ICS_SetTimeOutOffset)(TDeviceID dvid, uint32_t offsettime) = NULL;
double (WINAPI *GetQueryPerformanceCounter) (void) = NULL;
bool (WINAPI *ICS_TxPacket) (TDeviceID dvid, uint8_t id, TCommand cmd, const uint8_t *param, uint32_t len, TErrorCode *err) = NULL;
bool (WINAPI *ICS_RxPacket) (TDeviceID dvid, uint8_t *rdata, uint32_t rdatasize, uint32_t *rlen, TErrorCode *err) = NULL;
bool (WINAPI *ICS_ReadByteData)(TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdata, TErrorCode *errcode) = NULL;
bool (WINAPI *ICS_WriteByteData)(TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t dat, TErrorCode *errcode) = NULL;
bool (WINAPI *ICS_ReadWordData)(TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t *rdata, TErrorCode *errcode) = NULL;
bool (WINAPI *ICS_WriteWordData)(TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t dat, TErrorCode *errcode) = NULL;
bool (WINAPI *ICS_ReadBlockData)(TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdata, uint32_t len, TErrorCode *errcode) = NULL;
bool (WINAPI *ICS_WriteBlockData)(TDeviceID dvid, uint8_t id, uint16_t adr, const uint8_t *dat, uint32_t len, TErrorCode *errcode) = NULL;
bool (WINAPI *ICS_WriteSyncData) (TDeviceID dvid, uint8_t *dat, uint32_t size, TErrorCode *errcode) = NULL;
bool (WINAPI *ICS_Ping)(TDeviceID dvid, uint8_t id, TErrorCode *errcode) = NULL;

void (WINAPI *ics_purge) (TDeviceID dvid) = NULL;
bool (WINAPI *ics_puts) (TDeviceID dvid, uint8_t *c, int len) = NULL;
bool (WINAPI *ics_gets) (TDeviceID d, uint8_t *buf, int bytetoread, int *bytesread, double tout) = NULL;

bool (WINAPI *ICSL_SetLED) (TDeviceID dvid, uint8_t id, bool en) = NULL;
bool (WINAPI *ICSL_SetTorqueEnable) (TDeviceID dvid, uint8_t id, bool en) = NULL;
bool (WINAPI *ICSL_SetTorqueEnables) (TDeviceID dvid, const uint8_t *ids, bool *ens, int num) = NULL;
bool (WINAPI *ICSL_SetTorqueEnablesEquival) (TDeviceID dvid, const uint8_t *ids, int num, bool en) = NULL;
bool (WINAPI *ICSL_SetGoalAngle) (TDeviceID dvid, uint8_t id, double angle) = NULL;
bool (WINAPI *ICSL_SetGoalAngles) (TDeviceID dvid, const uint8_t *ids, double *angles, int num) = NULL;
bool (WINAPI *ICSL_GetPresentAngle) (TDeviceID dvid, uint8_t id, double *angle) = NULL;
bool (WINAPI *ICSL_GetPresentAngles) (TDeviceID dvid, const uint8_t *ids, double *angles, int num) = NULL;
bool (WINAPI *ICSL_SetGoalVelocity) (TDeviceID dvid, uint8_t id, double velocity) = NULL;
bool (WINAPI *ICSL_SetGoalVelocitys) (TDeviceID dvid, const uint8_t *ids, double *velocitys, int num) = NULL;
bool (WINAPI *ICSL_GetPresentVelocity) (TDeviceID dvid, uint8_t id, double *velocity) = NULL;
bool (WINAPI *ICSL_GetPresentVelocitys) (TDeviceID dvid, const uint8_t *ids, double *velocitys, int num) = NULL;
bool (WINAPI *ICSL_SetGoalAngleAndVelocity) (TDeviceID dvid, uint8_t id, double angle, double velocity) = NULL;
bool (WINAPI *ICSL_SetGoalAnglesAndVelocitys) (TDeviceID dvid, uint8_t *ids, PAngleVelocity anglevelocity, int num) = NULL;
bool (WINAPI *ICSL_SetGoalAngleAndTime) (TDeviceID dvid, uint8_t id, double angle, double sec) = NULL;
bool (WINAPI *ICSL_SetGoalAnglesAndTime) (TDeviceID dvid, const uint8_t *ids, double *angles, int num, double sec) = NULL;
bool (WINAPI *ICSL_SetGoalCurrent) (TDeviceID dvid, uint8_t id, double current) = NULL;
bool (WINAPI *ICSL_SetGoalCurrents) (TDeviceID dvid, const uint8_t *ids, double *currents, int num) = NULL;
bool (WINAPI *ICSL_GetPresentCurrent) (TDeviceID dvid, uint8_t id, double *current) = NULL;
bool (WINAPI *ICSL_GetPresentCurrents) (TDeviceID dvid, const uint8_t *ids, double *currents, int num) = NULL;
bool (WINAPI *ICSL_SetGoalPWM) (TDeviceID dvid, uint8_t id, double pwm) = NULL;
bool (WINAPI *ICSL_SetGoalPWMs) (TDeviceID dvid, const uint8_t *ids, double *pwms, int num) = NULL;
bool (WINAPI *ICSL_GetPresentPWM) (TDeviceID dvid, uint8_t id, double *pwm) = NULL;
bool (WINAPI *ICSL_GetPresentPWMs) (TDeviceID dvid, const uint8_t *ids, double *pwms, int num) = NULL;
bool (WINAPI *ICSL_SetOperatingMode) (TDeviceID dvid, uint8_t id, uint8_t mode) = NULL;
bool (WINAPI *ICSL_SetOperatingModesEquival) (TDeviceID dvid, const uint8_t *ids, int num, uint8_t mode) = NULL;
bool (WINAPI *ICSL_GetOperatingMode) (TDeviceID dvid, uint8_t id, uint8_t *mode) = NULL;
bool (WINAPI *ICSL_GetHWErrorCode) (TDeviceID dvid, uint8_t id, uint8_t *hwerr) = NULL;
TErrorCode (WINAPI *ICSL_GetErrorCode) (TDeviceID dvid, uint8_t id) = NULL;
PICSL_ModelInfo (WINAPI *ICSL_GetModelInfo) (TDeviceID dvid, uint8_t id) = NULL;
PICSL_ModelInfo (WINAPI *ICSL_ForceSetModelInfo) (TDeviceID dvid, uint8_t id, TICSL_DevType devtype) = NULL;
int (WINAPI *ICSL_ScanDevices) (TDeviceID dvid, uint8_t *ids) = NULL;
bool (WINAPI *ICSL_PrintDevicesList) (int *pf (const char *, ...)) = NULL;
void (WINAPI *ICSL_InitDevicesList) (void) = NULL;

static HMODULE hModule = NULL;

#ifdef _WIN32
  #ifdef _WIN64
  #define DLLNAME   "icslib_x64.dll"
  #else
  #define DLLNAME   "icslib_x32.dll"
  #endif
#else
  #ifdef __x86_64__
  #define DLLNAME   "icslib_x64.dll"
  #else
  #define DLLNAME   "icslib_x32.dll"
  #endif
#endif

// Load the DLL
bool LoadDLL (void) {
  hModule = LoadLibrary (DLLNAME);
  if (hModule == NULL) {
    return false;
  }

  ICS_OpenPort               = (void *)GetProcAddress (hModule, "ICS_OpenPort");
  ICS_ClosePort              = (void *)GetProcAddress (hModule, "ICS_ClosePort");
  ICS_SetEchoCancel          = (void *)GetProcAddress (hModule, "ICS_SetEchoCancel");
  ICS_SetBaudrate            = (void *)GetProcAddress (hModule, "ICS_SetBaudrate");
  ICS_Active                 = (void *)GetProcAddress (hModule, "ICS_Active");
  ICS_SetTimeOutOffset       = (void *)GetProcAddress (hModule, "ICS_SetTimeOutOffset");
  GetQueryPerformanceCounter = (void *)GetProcAddress (hModule, "GetQueryPerformanceCounter");
  ICS_TxPacket               = (void *)GetProcAddress (hModule, "ICS_TxPacket");
  ICS_RxPacket               = (void *)GetProcAddress (hModule, "ICS_RxPacket");
  ICS_ReadByteData           = (void *)GetProcAddress (hModule, "ICS_ReadByteData");
  ICS_WriteByteData          = (void *)GetProcAddress (hModule, "ICS_WriteByteData");
  ICS_ReadWordData           = (void *)GetProcAddress (hModule, "ICS_ReadWordData");
  ICS_WriteWordData          = (void *)GetProcAddress (hModule, "ICS_WriteWordData");
  ICS_ReadBlockData          = (void *)GetProcAddress (hModule, "ICS_ReadBlockData");
  ICS_WriteBlockData         = (void *)GetProcAddress (hModule, "ICS_WriteBlockData");
  ICS_WriteSyncData          = (void *)GetProcAddress (hModule, "ICS_WriteSyncData");
  ICS_Ping                   = (void *)GetProcAddress (hModule, "ICS_Ping");

  ics_purge                  = (void *)GetProcAddress (hModule, "ics_purge");
  ics_puts                   = (void *)GetProcAddress (hModule, "ics_puts");
  ics_gets                   = (void *)GetProcAddress (hModule, "ics_gets");

  ICSL_SetTorqueEnable           = (void *)GetProcAddress (hModule, "ICSL_SetTorqueEnable");
  ICSL_SetTorqueEnables          = (void *)GetProcAddress (hModule, "ICSL_SetTorqueEnables");
  ICSL_SetTorqueEnablesEquival   = (void *)GetProcAddress (hModule, "ICSL_SetTorqueEnablesEquival");
  ICSL_SetLED                    = (void *)GetProcAddress (hModule, "ICSL_SetLED");
  ICSL_SetGoalAngle              = (void *)GetProcAddress (hModule, "ICSL_SetGoalAngle");
  ICSL_SetGoalAngles             = (void *)GetProcAddress (hModule, "ICSL_SetGoalAngles");
  ICSL_GetPresentAngle           = (void *)GetProcAddress (hModule, "ICSL_GetPresentAngle");
  ICSL_GetPresentAngles          = (void *)GetProcAddress (hModule, "ICSL_GetPresentAngles");
  ICSL_SetGoalVelocity           = (void *)GetProcAddress (hModule, "ICSL_SetGoalVelocity");
  ICSL_SetGoalVelocitys          = (void *)GetProcAddress (hModule, "ICSL_SetGoalVelocitys");
  ICSL_GetPresentVelocity        = (void *)GetProcAddress (hModule, "ICSL_GetPresentVelocity");
  ICSL_GetPresentVelocitys       = (void *)GetProcAddress (hModule, "ICSL_GetPresentVelocitys");
  ICSL_SetGoalAngleAndVelocity   = (void *)GetProcAddress (hModule, "ICSL_SetGoalAngleAndVelocity");
  ICSL_SetGoalAnglesAndVelocitys = (void *)GetProcAddress (hModule, "ICSL_SetGoalAnglesAndVelocitys");
  ICSL_SetGoalAngleAndTime       = (void *)GetProcAddress (hModule, "ICSL_SetGoalAngleAndTime");
  ICSL_SetGoalAnglesAndTime      = (void *)GetProcAddress (hModule, "ICSL_SetGoalAnglesAndTime");
  ICSL_SetGoalPWM                = (void *)GetProcAddress (hModule, "ICSL_SetGoalPWM");
  ICSL_SetGoalPWMs               = (void *)GetProcAddress (hModule, "ICSL_SetGoalPWMs");
  ICSL_GetPresentPWM             = (void *)GetProcAddress (hModule, "ICSL_GetPresentPWM");
  ICSL_GetPresentPWMs            = (void *)GetProcAddress (hModule, "ICSL_GetPresentPWMs");
  ICSL_SetGoalCurrent            = (void *)GetProcAddress (hModule, "ICSL_SetGoalCurrent");
  ICSL_SetGoalCurrents           = (void *)GetProcAddress (hModule, "ICSL_SetGoalCurrents");
  ICSL_GetPresentCurrent         = (void *)GetProcAddress (hModule, "ICSL_GetPresentCurrent");
  ICSL_GetPresentCurrents        = (void *)GetProcAddress (hModule, "ICSL_GetPresentCurrents");
  ICSL_SetOperatingMode          = (void *)GetProcAddress (hModule, "ICSL_SetOperatingMode");
  ICSL_SetOperatingModesEquival  = (void *)GetProcAddress (hModule, "ICSL_SetOperatingModesEquival");
  ICSL_GetOperatingMode          = (void *)GetProcAddress (hModule, "ICSL_GetOperatingMode");
  ICSL_GetHWErrorCode            = (void *)GetProcAddress (hModule, "ICSL_GetHWErrorCode");
  ICSL_GetErrorCode              = (void *)GetProcAddress (hModule, "ICSL_GetErrorCode");
  ICSL_GetModelInfo              = (void *)GetProcAddress (hModule, "ICSL_GetModelInfo");
  ICSL_ForceSetModelInfo         = (void *)GetProcAddress (hModule, "ICSL_ForceSetModelInfo");
  ICSL_ScanDevices               = (void *)GetProcAddress (hModule, "ICSL_ScanDevices");
  ICSL_PrintDevicesList          = (void *)GetProcAddress (hModule, "ICSL_PrintDevicesList");
  ICSL_InitDevicesList           = (void *)GetProcAddress (hModule, "ICSL_InitDevicesList");

  return 
    (ICS_OpenPort               != NULL) &&
    (ICS_ClosePort              != NULL) &&
    (ICS_SetEchoCancel          != NULL) &&
    (ICS_SetBaudrate            != NULL) &&
    (ICS_Active                 != NULL) &&
    (ICS_SetTimeOutOffset       != NULL) &&
    (GetQueryPerformanceCounter != NULL) &&
    (ICS_TxPacket               != NULL) &&
    (ICS_RxPacket               != NULL) &&
    (ICS_ReadByteData           != NULL) &&
    (ICS_WriteByteData          != NULL) &&
    (ICS_ReadWordData           != NULL) &&
    (ICS_WriteWordData          != NULL) &&
    (ICS_ReadBlockData          != NULL) &&
    (ICS_WriteBlockData         != NULL) &&
    (ICS_WriteSyncData          != NULL) &&
    (ICS_Ping                   != NULL) &&

    (ics_purge                  != NULL) &&
    (ics_puts                   != NULL) &&
    (ics_gets                   != NULL) &&

    (ICSL_SetTorqueEnable           != NULL) &&
    (ICSL_SetTorqueEnables          != NULL) &&
    (ICSL_SetTorqueEnablesEquival   != NULL) &&
    (ICSL_SetLED                    != NULL) &&
    (ICSL_SetGoalAngle              != NULL) &&
    (ICSL_SetGoalAngles             != NULL) &&
    (ICSL_GetPresentAngle           != NULL) &&
    (ICSL_GetPresentAngles          != NULL) &&
    (ICSL_SetGoalVelocity           != NULL) &&
    (ICSL_SetGoalVelocitys          != NULL) &&
    (ICSL_GetPresentVelocity        != NULL) &&
    (ICSL_GetPresentVelocitys       != NULL) &&
    (ICSL_SetGoalAngleAndVelocity   != NULL) &&
    (ICSL_SetGoalAnglesAndVelocitys != NULL) &&
    (ICSL_SetGoalAngleAndTime       != NULL) &&
    (ICSL_SetGoalAnglesAndTime      != NULL) &&
    (ICSL_SetGoalCurrent            != NULL) &&
    (ICSL_SetGoalCurrents           != NULL) &&
    (ICSL_GetPresentCurrent         != NULL) &&
    (ICSL_GetPresentCurrents        != NULL) &&
    (ICSL_SetGoalPWM                != NULL) &&
    (ICSL_SetGoalPWMs               != NULL) &&
    (ICSL_GetPresentPWM             != NULL) &&
    (ICSL_GetPresentPWMs            != NULL) &&
    (ICSL_SetOperatingMode          != NULL) &&
    (ICSL_SetOperatingModesEquival  != NULL) &&
    (ICSL_GetOperatingMode          != NULL) &&
    (ICSL_GetHWErrorCode            != NULL) &&
    (ICSL_GetErrorCode              != NULL) &&
    (ICSL_GetModelInfo              != NULL) &&
    (ICSL_ForceSetModelInfo         != NULL) &&
    (ICSL_ScanDevices               != NULL) &&
    (ICSL_PrintDevicesList          != NULL) &&
    (ICSL_InitDevicesList           != NULL);
}

// Unload the DLL
void UnloadDLL (void) {
  if (hModule != NULL) {
    FreeLibrary (hModule);
    ICS_OpenPort               = NULL;
    ICS_ClosePort              = NULL;
    ICS_SetEchoCancel          = NULL;
    ICS_SetBaudrate            = NULL;
    ICS_Active                 = NULL;
    ICS_SetTimeOutOffset       = NULL;
    GetQueryPerformanceCounter = NULL;
    ICS_TxPacket               = NULL;
    ICS_RxPacket               = NULL;
    ICS_ReadByteData           = NULL;
    ICS_WriteByteData          = NULL;
    ICS_ReadWordData           = NULL;
    ICS_WriteWordData          = NULL;
    ICS_ReadBlockData          = NULL;
    ICS_WriteBlockData         = NULL;
    ICS_WriteSyncData          = NULL;
    ICS_Ping                   = NULL;

    ics_purge                  = NULL;
    ics_puts                   = NULL;
    ics_gets                   = NULL;

    ICSL_SetTorqueEnable           = NULL;
    ICSL_SetTorqueEnables          = NULL;
    ICSL_SetTorqueEnablesEquival   = NULL;
    ICSL_SetLED                    = NULL;
    ICSL_SetGoalAngle              = NULL;
    ICSL_SetGoalAngles             = NULL;
    ICSL_GetPresentAngle           = NULL;
    ICSL_GetPresentAngles          = NULL;
    ICSL_SetGoalVelocity           = NULL;
    ICSL_SetGoalVelocitys          = NULL;
    ICSL_GetPresentVelocity        = NULL;
    ICSL_GetPresentVelocitys       = NULL;
    ICSL_SetGoalAngleAndVelocity   = NULL;
    ICSL_SetGoalAnglesAndVelocitys = NULL;
    ICSL_SetGoalAngleAndTime       = NULL;
    ICSL_SetGoalAnglesAndTime      = NULL;
    ICSL_SetGoalCurrent            = NULL;
    ICSL_SetGoalCurrents           = NULL;
    ICSL_GetPresentCurrent         = NULL;
    ICSL_GetPresentCurrents        = NULL;
    ICSL_SetGoalPWM                = NULL;
    ICSL_SetGoalPWMs               = NULL;
    ICSL_GetPresentPWM             = NULL;
    ICSL_GetPresentPWMs            = NULL;
    ICSL_SetOperatingMode          = NULL;
    ICSL_SetOperatingModesEquival  = NULL;
    ICSL_GetOperatingMode          = NULL;
    ICSL_GetHWErrorCode            = NULL;
    ICSL_GetErrorCode              = NULL;
    ICSL_GetModelInfo              = NULL;
    ICSL_ForceSetModelInfo         = NULL;
    ICSL_ScanDevices               = NULL;
    ICSL_PrintDevicesList          = NULL;
    ICSL_InitDevicesList           = NULL;
  }
}

#endif

#ifdef __cplusplus
}
#endif

#endif //_ICSLIB_H_INCLUDE
